package ahc.collection;

import java.util.Map;
import java.util.HashMap;


/**
 * This abstract class serves as a generic base for a cache that returns an instance based on a parameter. The
 *  first time the get method is called for a parameter, the corresponding instance is created by calling
 *  the abstract createNew method, for subsequent calls with the same parameter (using equals), the cached
 *  instance that was created on the first call is returned.
 *
 * @author Arno Haase
 */
public abstract class Cache {
    private final Map _cache = new HashMap ();

    public Object get (Object key) {
        Object result = _cache.get (key);

        if (result == null) {
            result = createNew (key);
            _cache.put (key, result);
        }

        return result;
    }

    protected abstract Object createNew (Object key);
}
