package ahc.collection;

import ahc.collection.equality.EqualityStrategy;
import ahc.collection.Collection;


/**
 * This is the generic, configurable implementation of a MultiMap.
 *
 * @author Arno Haase
 */
public class GenericMultiMap implements MultiMap {
    private final Map _map;
    private int _size = 0;

    public GenericMultiMap (Map map) {
        if (! map.isEmpty())
            throw new IllegalArgumentException ("map must be empty");
        _map = map;
    }

    public EqualityStrategy getValueEqualityStrategy () {
        return _map.getValueEqualityStrategy ();
    }

    public int getCount (Object key) {
        final List l = getAll (key);
        return (l == null) ? 0 : l.size ();
    }

    public List getAll (Object key) {
        final List result = (List) get (key);
        return (result != null) ? result : createEmptyList();
    }

    public int size () {
        return _size;
    }

    public boolean isEmpty () {
        return _map.isEmpty ();
    }

    public boolean containsKey (Object key) {
        return _map.containsKey (key);
    }

    public boolean containsValue (Object value) {
        return vals ().contains (value);
    }

    public Object get (Object key) {
        return _map.get (key);
    }

    private List createEmptyList () {
        return ListHelper.createArrayList (getValueEqualityStrategy (), ListHelper.ACCESS_TRANSPARENT);
    }

    public Object put (Object key, Object value) {
        List l = (List) _map.get (key);
        if (l == null) {
            l = createEmptyList ();
            _map.put (key, l);
        }

        l.append (value);
        _size++;
        return null;
    }

    public Object remove (Object key) {
        _size -= getCount (key);

        return _map.remove (key);
    }

    public boolean remove (Object key, Object value) {
        final List l = getAll (key);
        if (l == null)
            return false;

        _size--;
        return l.remove (value);
    }

    public void putAll (java.util.Map t) {
        for (java.util.Iterator iter = t.keySet ().iterator (); iter.hasNext ();) {
            final Object curKey = iter.next ();
            put (curKey, t.get (curKey));
        }
    }

    public void putAll (MultiMap m) {
        for (java.util.Iterator iter = m.keySet ().iterator (); iter.hasNext ();) {
            final Object curKey = iter.next ();
            for (Iterator iter2 = m.getAll (curKey).begin (); iter2.hasCurrent (); iter2.next ()) {
                put (curKey, iter2.current ());
            }
        }
    }

    public void clear () {
        _map.clear ();
        _size = 0;
    }

    public Set keys () {
        return _map.keys ();
    }

    public Collection vals () {
        final List result = ListHelper.createArrayList (getValueEqualityStrategy (), ListHelper.ACCESS_TRANSPARENT);
        for (Iterator iter = _map.keys ().begin (); iter.hasCurrent (); iter.next ()) {
            result.addAll (getAll (iter.current ()));
        }
        return result;
    }

    public Set entries () {
        return _map.entries ();
    }

    public java.util.Set keySet () {
        return keys ();
    }

    public java.util.Collection values () {
        return vals ();
    }

    public java.util.Set entrySet () {
        return entries ();
    }
}
