package ahc.collection;

import ahc.collection.access.AccessStrategy;
import ahc.collection.access.GenericAccessStrategy;
import ahc.collection.access.TransparentAccessStrategy;
import ahc.collection.equality.EqualityStrategy;
import ahc.collection.equality.EqualsEqualityStrategy;
import ahc.collection.equality.IdentityEqualityStrategy;
import ahc.collection.store.ArrayStore;
import ahc.collection.store.DlinkedStore;
import ahc.collection.store.Store;
import ahc.collection.GenericCollection;
import ahc.collection.List;

import java.util.Comparator;


/**
 * This class provides convenience methods for the creation and management of List instances.
 *
 * @author Arno Haase
 */
public class ListHelper {
    public static final EqualityStrategy EQ_EQUALS = new EqualsEqualityStrategy ();
    public static final EqualityStrategy EQ_IDENTITY = new IdentityEqualityStrategy ();

    public static final AccessStrategy ACCESS_TRANSPARENT = TransparentAccessStrategy.INSTANCE;



    public static List synchronizedList (List list) {
        return SynchronizedList.create (list);
    }

    //-----------------------------------------------------------------------------------

    public static List createList () {
        return createArrayList();
    }


    public static List createArrayList () {
        return createArrayList (EQ_EQUALS, ACCESS_TRANSPARENT);
    }

    public static List createArrayList (int initialSize) {
        return createArrayList (EQ_EQUALS, ACCESS_TRANSPARENT, initialSize);
    }

    public static List createArrayList (Comparator comp) {
        return createArrayList (comp, false);
    }

    public static List createArrayList (Comparator comp, int initialSize) {
        return createArrayList (comp, false, initialSize);
    }

    public static List createArrayList (Comparator comp, boolean ensureUniqueness) {
        return createArrayList (EQ_EQUALS, GenericAccessStrategy.create (comp, ensureUniqueness));
    }

    public static List createArrayList (Comparator comp, boolean ensureUniqueness, int initialSize) {
        return createArrayList (EQ_EQUALS, GenericAccessStrategy.create (comp, ensureUniqueness), initialSize);
    }

    public static List createArrayList (EqualityStrategy eqStrat, AccessStrategy accessStrat) {
        return createList (new ArrayStore (eqStrat), accessStrat);
    }

    public static List createArrayList (EqualityStrategy eqStrat, AccessStrategy accessStrat, int initialSize) {
        return createList (new ArrayStore (eqStrat, initialSize), accessStrat);
    }


    public static List createDlinkedList () {
        return createDlinkedList (EQ_EQUALS, ACCESS_TRANSPARENT);
    }

    public static List createDlinkedList (Comparator comp) {
        return createDlinkedList (comp, false);
    }

    public static List createDlinkedList (Comparator comp, boolean ensureUniqueness) {
        return createDlinkedList (EQ_EQUALS, GenericAccessStrategy.create (comp, ensureUniqueness));
    }

    public static List createDlinkedList (EqualityStrategy eqStrat, AccessStrategy accessStrat) {
        return createList (new DlinkedStore (eqStrat), accessStrat);
    }


    public static List createList (Store store, AccessStrategy accessStrategy) {
        return new GenericCollection (store, accessStrategy);
    }

    //todo: unmodifiable, synchronized, sort, binarySearch, shuffle
    //todo: copy in all variations
}
