package ahc.collection;

import ahc.collection.equality.EqualityStrategy;
import ahc.collection.equality.EqualsEqualityStrategy;
import ahc.collection.equality.IdentityEqualityStrategy;
import ahc.collection.access.AccessStrategy;
import ahc.collection.access.GenericAccessStrategy;
import ahc.collection.store.ArrayStore;
import ahc.collection.store.DlinkedStore;
import ahc.collection.store.HashStore;
import ahc.collection.store.Store;
import ahc.collection.GenericMap;
import ahc.collection.GenericMultiMap;
import ahc.collection.Map;
import ahc.collection.MultiMap;

import java.util.Comparator;


/**
 * This class provides convenience methods for the creation and management of MultiMaps.
 *
 * @author Arno Haase
 */
public class MultiMapHelper {
    public static final EqualityStrategy EQ_EQUALS = new EqualsEqualityStrategy ();
    public static final EqualityStrategy EQ_IDENTITY = new IdentityEqualityStrategy ();

    private static final AccessStrategy ACCESS_UNIQUE = GenericAccessStrategy.create (null, true);


    //----------------------- createMultiMap -----------------------------

    public static MultiMap createMultiMap () {
        return createHashMultiMap ();
    }

    //----------------------- createArrayMultiMap ------------------------


    public static MultiMap createArrayMultiMap () {
        return createArrayMultiMap (EQ_EQUALS, ACCESS_UNIQUE);
    }

    public static MultiMap createArrayMultiMap (int initialSize) {
        return createArrayMultiMap (EQ_EQUALS, ACCESS_UNIQUE, initialSize);
    }

    public static MultiMap createArrayMultiMap (EqualityStrategy eqStrat) {
        return createArrayMultiMap (eqStrat, ACCESS_UNIQUE);
    }

    public static MultiMap createArrayMultiMap (EqualityStrategy eqStrat, int initialSize) {
        return createArrayMultiMap (eqStrat, ACCESS_UNIQUE, initialSize);
    }

    public static MultiMap createArrayMultiMap (Comparator comp) {
        return createArrayMultiMap (EQ_EQUALS, GenericAccessStrategy.create (new EntryKeyComparator(comp), true));
    }

    public static MultiMap createArrayMultiMap (Comparator comp, int initialSize) {
        return createArrayMultiMap (EQ_EQUALS, GenericAccessStrategy.create (new EntryKeyComparator(comp), true), initialSize);
    }

    public static MultiMap createArrayMultiMap (AccessStrategy accessStrat) {
        return createArrayMultiMap (EQ_EQUALS, accessStrat);
    }

    public static MultiMap createArrayMultiMap (AccessStrategy accessStrat, int initialSize) {
        return createArrayMultiMap (EQ_EQUALS, accessStrat, initialSize);
    }

    public static MultiMap createArrayMultiMap (EqualityStrategy eqStrat, AccessStrategy accessStrat) {
        return createMultiMap (new ArrayStore (eqStrat), accessStrat);
    }

    public static MultiMap createArrayMultiMap (EqualityStrategy eqStrat, AccessStrategy accessStrat, int initialSize) {
        return createMultiMap (new ArrayStore (eqStrat, initialSize), accessStrat);
    }


    //----------------------- createDlinkedMultiMap ------------------------


    public static MultiMap createDlinkedMultiMap () {
        return createDlinkedMultiMap (EQ_EQUALS, ACCESS_UNIQUE);
    }

    public static MultiMap createDlinkedMultiMap (EqualityStrategy eqStrat) {
        return createDlinkedMultiMap (eqStrat, ACCESS_UNIQUE);
    }

    public static MultiMap createDlinkedMultiMap (Comparator comp) {
        return createDlinkedMultiMap (EQ_EQUALS, GenericAccessStrategy.create (new EntryKeyComparator(comp), true));
    }

    public static MultiMap createDlinkedMultiMap (AccessStrategy accessStrat) {
        return createDlinkedMultiMap (EQ_EQUALS, accessStrat);
    }

    public static MultiMap createDlinkedMultiMap (EqualityStrategy eqStrat, AccessStrategy accessStrat) {
        return createMultiMap (new DlinkedStore (eqStrat), accessStrat);
    }


    //----------------------- createHashMultiMap ------------------------


    public static MultiMap createHashMultiMap () {
        return createHashMultiMap (EQ_EQUALS);
    }

    public static MultiMap createHashMultiMap (int initialSize) {
        return createHashMultiMap (EQ_EQUALS, initialSize);
    }

    public static MultiMap createHashMultiMap (int initialSize, double loadFactor) {
        return createHashMultiMap (EQ_EQUALS, initialSize, loadFactor);
    }

    public static MultiMap createHashMultiMap (EqualityStrategy eqStrat) {
        return createHashMultiMap (eqStrat, ACCESS_UNIQUE);
    }

    public static MultiMap createHashMultiMap (EqualityStrategy eqStrat, int initialSize) {
        return createHashMultiMap (eqStrat, ACCESS_UNIQUE, initialSize);
    }

    public static MultiMap createHashMultiMap (EqualityStrategy eqStrat, int initialSize, double loadFactor) {
        return createHashMultiMap (eqStrat, ACCESS_UNIQUE, initialSize, loadFactor);
    }

    public static MultiMap createHashMultiMap (AccessStrategy accessStrat) {
        return createHashMultiMap (EQ_EQUALS, accessStrat);
    }

    public static MultiMap createHashMultiMap (AccessStrategy accessStrat, int initialSize) {
        return createHashMultiMap (EQ_EQUALS, accessStrat, initialSize);
    }

    public static MultiMap createHashMultiMap (AccessStrategy accessStrat, int initialSize, double loadFactor) {
        return createHashMultiMap (EQ_EQUALS, accessStrat, initialSize, loadFactor);
    }

    public static MultiMap createHashMultiMap (EqualityStrategy eqStrat, AccessStrategy accessStrat) {
        return createMultiMap (new HashStore (eqStrat), accessStrat);
    }

    public static MultiMap createHashMultiMap (EqualityStrategy eqStrat, AccessStrategy accessStrat, int initialSize) {
        return createMultiMap (new HashStore (eqStrat, initialSize), accessStrat);
    }

    public static MultiMap createHashMultiMap (EqualityStrategy eqStrat, AccessStrategy accessStrat, int initialSize, double loadFactor) {
        return createMultiMap (new HashStore (eqStrat, initialSize, loadFactor), accessStrat);
    }


    //------------------- generic createMultiMap ------------------------------


    private static MultiMap createMultiMap (Store store, AccessStrategy accessStrat) {
        return createMultiMap (new GenericMap (store, accessStrat));
    }


    public static MultiMap createMultiMap (Map map) {
        return new GenericMultiMap (map);
    }
}
