package ahc.collection;

import ahc.collection.access.AccessStrategy;
import ahc.collection.access.GenericAccessStrategy;
import ahc.collection.equality.EqualityStrategy;
import ahc.collection.equality.EqualsEqualityStrategy;
import ahc.collection.equality.IdentityEqualityStrategy;
import ahc.collection.store.Store;
import ahc.collection.store.HashStore;
import ahc.collection.store.DlinkedStore;
import ahc.collection.store.ArrayStore;
import ahc.collection.GenericMap;
import ahc.collection.GenericMultiSet;
import ahc.collection.Map;
import ahc.collection.MultiSet;

import java.util.Comparator;


/**
 * This class contains convenience methods for the creation and management of MultiSets.
 *
 * @author Arno Haase
 */
public class MultiSetHelper {
    public static final EqualityStrategy EQ_EQUALS = new EqualsEqualityStrategy ();
    public static final EqualityStrategy EQ_IDENTITY = new IdentityEqualityStrategy ();

    private static final AccessStrategy ACCESS_UNIQUE = GenericAccessStrategy.create (null, true);


    //----------------------- createMap -----------------------------

    public static MultiSet createMultiSet () {
        return createHashMultiSet ();
    }

    //----------------------- createArrayMap ------------------------


    public static MultiSet createArrayMultiSet () {
        return createArrayMultiSet (EQ_EQUALS, ACCESS_UNIQUE);
    }

    public static MultiSet createArrayMultiSet (int initialSize) {
        return createArrayMultiSet (EQ_EQUALS, ACCESS_UNIQUE, initialSize);
    }

    public static MultiSet createArrayMultiSet (EqualityStrategy eqStrat) {
        return createArrayMultiSet (eqStrat, ACCESS_UNIQUE);
    }

    public static MultiSet createArrayMultiSet (EqualityStrategy eqStrat, int initialSize) {
        return createArrayMultiSet (eqStrat, ACCESS_UNIQUE, initialSize);
    }

    public static MultiSet createArrayMultiSet (Comparator comp) {
        return createArrayMultiSet (EQ_EQUALS, GenericAccessStrategy.create (new EntryKeyComparator (comp), true));
    }

    public static MultiSet createArrayMultiSet (Comparator comp, int initialSize) {
        return createArrayMultiSet (EQ_EQUALS, GenericAccessStrategy.create (new EntryKeyComparator (comp), true), initialSize);
    }

    public static MultiSet createArrayMultiSet (AccessStrategy accessStrat) {
        return createArrayMultiSet (EQ_EQUALS, accessStrat);
    }

    public static MultiSet createArrayMultiSet (AccessStrategy accessStrat, int initialSize) {
        return createArrayMultiSet (EQ_EQUALS, accessStrat, initialSize);
    }

    public static MultiSet createArrayMultiSet (EqualityStrategy eqStrat, AccessStrategy accessStrat) {
        return createMultiSet (new ArrayStore (eqStrat), accessStrat);
    }

    public static MultiSet createArrayMultiSet (EqualityStrategy eqStrat, AccessStrategy accessStrat, int initialSize) {
        return createMultiSet (new ArrayStore (eqStrat, initialSize), accessStrat);
    }


    //----------------------- createDlinkedMap ------------------------


    public static MultiSet createDlinkedMultiSet () {
        return createDlinkedMultiSet (EQ_EQUALS, ACCESS_UNIQUE);
    }

    public static MultiSet createDlinkedMultiSet (EqualityStrategy eqStrat) {
        return createDlinkedMultiSet (eqStrat, ACCESS_UNIQUE);
    }

    public static MultiSet createDlinkedMultiSet (Comparator comp) {
        return createDlinkedMultiSet (EQ_EQUALS, GenericAccessStrategy.create (new EntryKeyComparator (comp), true));
    }

    public static MultiSet createDlinkedMultiSet (AccessStrategy accessStrat) {
        return createDlinkedMultiSet (EQ_EQUALS, accessStrat);
    }

    public static MultiSet createDlinkedMultiSet (EqualityStrategy eqStrat, AccessStrategy accessStrat) {
        return createMultiSet (new DlinkedStore (eqStrat), accessStrat);
    }


    //----------------------- createHashMap ------------------------


    public static MultiSet createHashMultiSet () {
        return createHashMultiSet (EQ_EQUALS);
    }

    public static MultiSet createHashMultiSet (int initialSize) {
        return createHashMultiSet (EQ_EQUALS, initialSize);
    }

    public static MultiSet createHashMultiSet (int initialSize, double loadFactor) {
        return createHashMultiSet (EQ_EQUALS, initialSize, loadFactor);
    }

    public static MultiSet createHashMultiSet (EqualityStrategy eqStrat) {
        return createHashMultiSet (eqStrat, ACCESS_UNIQUE);
    }

    public static MultiSet createHashMultiSet (EqualityStrategy eqStrat, int initialSize) {
        return createHashMultiSet (eqStrat, ACCESS_UNIQUE, initialSize);
    }

    public static MultiSet createHashMultiSet (EqualityStrategy eqStrat, int initialSize, double loadFactor) {
        return createHashMultiSet (eqStrat, ACCESS_UNIQUE, initialSize, loadFactor);
    }

    public static MultiSet createHashMultiSet (AccessStrategy accessStrat) {
        return createHashMultiSet (EQ_EQUALS, accessStrat);
    }

    public static MultiSet createHashMultiSet (AccessStrategy accessStrat, int initialSize) {
        return createHashMultiSet (EQ_EQUALS, accessStrat, initialSize);
    }

    public static MultiSet createHashMultiSet (AccessStrategy accessStrat, int initialSize, double loadFactor) {
        return createHashMultiSet (EQ_EQUALS, accessStrat, initialSize, loadFactor);
    }

    public static MultiSet createHashMultiSet (EqualityStrategy eqStrat, AccessStrategy accessStrat) {
        return createMultiSet (new HashStore (eqStrat), accessStrat);
    }

    public static MultiSet createHashMultiSet (EqualityStrategy eqStrat, AccessStrategy accessStrat, int initialSize) {
        return createMultiSet (new HashStore (eqStrat, initialSize), accessStrat);
    }

    public static MultiSet createHashMultiSet (EqualityStrategy eqStrat, AccessStrategy accessStrat, int initialSize, double loadFactor) {
        return createMultiSet (new HashStore (eqStrat, initialSize, loadFactor), accessStrat);
    }


    //----------------------- generic createMultiSet ------------------------


    public static MultiSet createMultiSet (Store store, AccessStrategy accessStrat) {
        return createMultiSet (new GenericMap (store, accessStrat));
    }

    public static MultiSet createMultiSet (Map map) {
        return new GenericMultiSet (map);
    }

    //todo: synchronized
}
