package ahc.collection;

import ahc.util.NaturalOrderComparator;
import ahc.collection.Pair;

import java.util.Comparator;


/**
 * This is a very special comparator implementation that is designed for use in priority collections.
 *  It assumes that all compared elements are Pairs, and it compares them applying an inner
 *  comparator to the first element of the pair.
 *
 * @author Arno Haase
 */
class PriorityComparator implements Comparator {
    private final Comparator _inner;


    public PriorityComparator () {
        _inner = new NaturalOrderComparator ();
    }

    public PriorityComparator (Comparator inner) {
        _inner = inner;
    }


    public int compare (Object o1, Object o2) {
        final Pair p1 = (Pair) o1;
        final Pair p2 = (Pair) o2;
        return _inner.compare (p1.getFirst (), p2.getFirst ());
    }
}
