package ahc.collection;

import ahc.exception.TimeoutException;


/**
 * This is the generic interface that all queue classes must implement.
 *
 * @author Arno Haase
 */
public interface Queue {
    void add (Object obj);

    /**
     * This method is implemented only for non-threadsafe queues so that it does
     *  not have InterruptedException in its signature. Therefore, this method does
     *  not block but rather throws NoSuchElementException if called on an empty
     *  queue.
     */
    Object remove ();
    Object remove (boolean blocking) throws InterruptedException;
    Object remove (int timeoutMillis) throws InterruptedException, TimeoutException;

    /**
     * This method is implemented only for non-threadsafe queues so that it does
     *  not have InterruptedException in its signature. Therefore, this method does
     *  not block but rather throws NoSuchElementException if called on an empty
     *  queue.
     */
    Object peek ();
    Object peek (boolean blocking) throws InterruptedException;
    Object peek (int timeoutMillis) throws InterruptedException, TimeoutException;

    int size ();
    boolean isEmpty ();
    void clear ();

    boolean isThreadSafe ();
}
