package ahc.collection;

import ahc.collection.store.Store;
import ahc.collection.store.DlinkedStore;
import ahc.collection.access.AccessStrategy;
import ahc.collection.access.TransparentAccessStrategy;
import ahc.collection.equality.EqualityStrategy;
import ahc.collection.equality.EqualsEqualityStrategy;
import ahc.collection.GenericQueue;
import ahc.collection.Queue;


/**
 * This class provides convenience methods for creating and managing queue instances.
 *
 * <p> Synchronization is provided for queues in a different way than for other collections: Rather than
 *  wrapping non-synchronized queues, thread-safety is an integral feature of queues that needs to
 *  be selected on creation. This is an optimization that takes the typical situation into account that
 *  one set of threads pushes and other threads wait on the retrieving end.
 *
 * @author Arno Haase
 */
public class QueueHelper {
    public static final AccessStrategy ACCESS_TRANSPARENT = TransparentAccessStrategy.INSTANCE;

    private static final EqualityStrategy EQ_EQUALS = new EqualsEqualityStrategy();


    //------------------------------ DlinkedList-based createQueue --------------------------


    public static Queue createQueue (boolean isThreadSafe) {
        return createQueue (new DlinkedStore(EQ_EQUALS), ACCESS_TRANSPARENT, isThreadSafe);
    }


    //------------------------------- generic createQueue ----------------------------


    public static Queue createQueue (Store store, AccessStrategy accessStrategy, boolean isThreadSafe) {
        return new GenericQueue (store, accessStrategy, isThreadSafe);
    }
}
