package ahc.collection;

import ahc.collection.access.AccessStrategy;
import ahc.collection.access.GenericAccessStrategy;
import ahc.collection.equality.EqualityStrategy;
import ahc.collection.equality.EqualsEqualityStrategy;
import ahc.collection.equality.IdentityEqualityStrategy;
import ahc.collection.store.ArrayStore;
import ahc.collection.store.DlinkedStore;
import ahc.collection.store.HashStore;
import ahc.collection.store.Store;
import ahc.collection.GenericCollection;
import ahc.collection.Set;

import java.util.Comparator;


/**
 * This class provides convenience methods for the creation and management of Set instances.
 *
 * @author Arno Haase
 */
public class SetHelper {
    public static final EqualityStrategy EQ_EQUALS = new EqualsEqualityStrategy ();
    public static final EqualityStrategy EQ_IDENTITY = new IdentityEqualityStrategy ();

    public static final AccessStrategy ACCESS_DEFAULT = GenericAccessStrategy.create (null, true);


    public static Set createSet () {
        return createHashSet ();
    }


    public static Set createArraySet () {
        return createArraySet (null);
    }

    public static Set createArraySet (int initialSize) {
        return createArraySet (null, initialSize);
    }

    public static Set createArraySet (Comparator comp) {
        return createArraySet (EQ_EQUALS, comp);
    }

    public static Set createArraySet (Comparator comp, int initialSize) {
        return createArraySet (EQ_EQUALS, comp, initialSize);
    }

    public static Set createArraySet (EqualityStrategy eqStrat, Comparator comp) {
        return createArraySet (eqStrat, GenericAccessStrategy.create (comp, true));
    }

    public static Set createArraySet (EqualityStrategy eqStrat, Comparator comp, int initialSize) {
        return createArraySet (eqStrat, GenericAccessStrategy.create (comp, true), initialSize);
    }

    public static Set createArraySet (EqualityStrategy eqStrat, AccessStrategy accessStrat) {
        return createSet (new ArrayStore (eqStrat), accessStrat);
    }

    public static Set createArraySet (EqualityStrategy eqStrat, AccessStrategy accessStrat, int initialSize) {
        return createSet (new ArrayStore (eqStrat, initialSize), accessStrat);
    }


    public static Set createDlinkedSet () {
        return createDlinkedSet (null);
    }

    public static Set createDlinkedSet (Comparator comp) {
        return createDlinkedSet (EQ_EQUALS, comp);
    }

    public static Set createDlinkedSet (EqualityStrategy eqStrat, Comparator comp) {
        return createDlinkedSet (eqStrat, GenericAccessStrategy.create (comp, true));
    }

    public static Set createDlinkedSet (EqualityStrategy eqStrat, AccessStrategy accessStrat) {
        return createSet (new DlinkedStore (eqStrat), accessStrat);
    }


    public static Set createHashSet () {
        return createHashSet (EQ_EQUALS);
    }

    public static Set createHashSet (int initialSize) {
        return createHashSet (EQ_EQUALS, initialSize);
    }

    public static Set createHashSet (int initialSize, double loadFactor) {
        return createHashSet (EQ_EQUALS, initialSize, loadFactor);
    }

    public static Set createHashSet (EqualityStrategy eqStrat) {
        return createHashSet (eqStrat, ACCESS_DEFAULT);
    }

    public static Set createHashSet (EqualityStrategy eqStrat, int initialSize) {
        return createHashSet (eqStrat, ACCESS_DEFAULT, initialSize);
    }

    public static Set createHashSet (EqualityStrategy eqStrat, int initialSize, double loadFactor) {
        return createHashSet (eqStrat, ACCESS_DEFAULT, initialSize, loadFactor);
    }

    public static Set createHashSet (EqualityStrategy eqStrat, AccessStrategy accessStrat) {
        return createSet (new HashStore (eqStrat), accessStrat);
    }

    public static Set createHashSet (EqualityStrategy eqStrat, AccessStrategy accessStrat, int initialSize) {
        return createSet (new HashStore (eqStrat, initialSize), accessStrat);
    }

    public static Set createHashSet (EqualityStrategy eqStrat, AccessStrategy accessStrat, int initialSize, double loadFactor) {
        return createSet (new HashStore (eqStrat, initialSize, loadFactor), accessStrat);
    }


    public static Set createSet (Store store, AccessStrategy accessStrategy) {
        return new GenericCollection (store, accessStrategy);
    }

    //todo synchronized set
}
