package ahc.collection;


/**
 * @author Arno Haase
 */
class SynchronizedIterator implements Iterator {
    private final Iterator _inner;
    private final Object _lock;

    public static SynchronizedIterator create (Iterator inner, Collection lock) {
        if (inner == null)
            return null;

        if (lock == null)
            throw new IllegalArgumentException ("Lock must not be null");

        return new SynchronizedIterator (inner, lock);
    }

    private SynchronizedIterator (Iterator inner, Collection lock) {
        _inner = inner;
        _lock = lock;
    }


    public boolean hasCurrent () {
        synchronized (_lock) {
            return _inner.hasCurrent ();
        }
    }

    public Object current () {
        synchronized (_lock) {
            return _inner.current ();
        }
    }

    public int currentIndex () {
        synchronized (_lock) {
            return _inner.currentIndex ();
        }
    }

    public Object next (int n) {
        synchronized (_lock) {
            return _inner.next (n);
        }
    }

    public Object previous (int n) {
        synchronized (_lock) {
            return _inner.previous (n);
        }
    }

    public boolean replace (Object o) {
        synchronized (_lock) {
            return _inner.replace (o);
        }
    }

    public boolean hasNext () {
        synchronized (_lock) {
            return _inner.hasNext ();
        }
    }

    public Object next () {
        synchronized (_lock) {
            return _inner.next ();
        }
    }

    public boolean hasPrevious () {
        synchronized (_lock) {
            return _inner.hasPrevious ();
        }
    }

    public Object previous () {
        synchronized (_lock) {
            return _inner.previous ();
        }
    }

    public int nextIndex () {
        synchronized (_lock) {
            return _inner.nextIndex ();
        }
    }

    public int previousIndex () {
        synchronized (_lock) {
            return _inner.previousIndex ();
        }
    }

    public void remove () {
        synchronized (_lock) {
            _inner.remove ();
        }
    }

    public void set (Object o) {
        synchronized (_lock) {
            _inner.set (o);
        }
    }

    public void add (Object o) {
        synchronized (_lock) {
            _inner.add (o);
        }
    }
}
