package ahc.collection;


/**
 * @author Arno Haase
 */
class SynchronizedLegacyIterator implements java.util.Iterator {
    private final java.util.Iterator _inner;
    private final Object _lock;

    public static SynchronizedLegacyIterator create (java.util.Iterator inner, java.util.Collection lock) {
        if (inner == null)
            return null;

        if (lock == null)
            throw new IllegalArgumentException ("Lock must not be null");

        return new SynchronizedLegacyIterator (inner, lock);
    }

    private SynchronizedLegacyIterator (java.util.Iterator inner, java.util.Collection lock) {
        _inner = inner;
        _lock = lock;
    }


    public boolean hasNext () {
        synchronized (_lock) {
            return _inner.hasNext();
        }
    }

    public Object next () {
        synchronized (_lock) {
            return _inner.next();
        }
    }

    public void remove () {
        synchronized (_lock) {
            _inner.remove ();
        }
    }
}
