package ahc.collection;

import java.util.Iterator;
import java.util.Collection;
import java.util.ListIterator;
import java.util.List;


/**
 * @author Arno Haase
 */
class SynchronizedLegacyList implements List {
    private final List _inner;
    private final List _lock;


    public static SynchronizedLegacyList create (List inner, List lock) {
        if (inner == null)
            return null;

        if (lock == null)
            throw new IllegalArgumentException ("Lock must not be null");

        return new SynchronizedLegacyList (inner, lock);
    }


    private SynchronizedLegacyList (List inner, List lock) {
        _inner = inner;
        _lock = lock;
    }

    public int size () {
        synchronized (_lock) {
            return _inner.size ();
        }
    }

    public boolean isEmpty () {
        synchronized (_lock) {
            return _inner.isEmpty ();
        }
    }

    public boolean contains (Object o) {
        synchronized (_lock) {
            return _inner.contains (o);
        }
    }

    public Iterator iterator () {
        synchronized (_lock) {
            return SynchronizedLegacyIterator.create (_inner.iterator (), _lock);
        }
    }

    public Object[] toArray () {
        synchronized (_lock) {
            return _inner.toArray ();
        }
    }

    public Object[] toArray (Object a[]) {
        synchronized (_lock) {
            return _inner.toArray (a);
        }
    }

    public boolean add (Object o) {
        synchronized (_lock) {
            return _inner.add (o);
        }
    }

    public boolean remove (Object o) {
        synchronized (_lock) {
            return _inner.remove (o);
        }
    }

    public boolean containsAll (Collection c) {
        synchronized (_lock) {
            return _inner.containsAll (c);
        }
    }

    public boolean addAll (Collection c) {
        synchronized (_lock) {
            return _inner.addAll (c);
        }
    }

    public boolean addAll (int index, Collection c) {
        synchronized (_lock) {
            return _inner.addAll (index, c);
        }
    }

    public boolean removeAll (Collection c) {
        synchronized (_lock) {
            return _inner.removeAll (c);
        }
    }

    public boolean retainAll (Collection c) {
        synchronized (_lock) {
            return _inner.retainAll (c);
        }
    }

    public void clear () {
        synchronized (_lock) {
            _inner.clear ();
        }
    }

    public Object get (int index) {
        synchronized (_lock) {
            return _inner.get (index);
        }
    }

    public Object set (int index, Object element) {
        synchronized (_lock) {
            return _inner.set (index, element);
        }
    }

    public void add (int index, Object element) {
        synchronized (_lock) {
            _inner.add (index, element);
        }
    }

    public Object remove (int index) {
        synchronized (_lock) {
            return _inner.remove (index);
        }
    }

    public int indexOf (Object o) {
        synchronized (_lock) {
            return _inner.indexOf (o);
        }
    }

    public int lastIndexOf (Object o) {
        synchronized (_lock) {
            return _inner.lastIndexOf (o);
        }
    }

    public ListIterator listIterator () {
        synchronized (_lock) {
            return SynchronizedLegacyListIterator.create (_inner.listIterator (), _lock);
        }
    }

    public ListIterator listIterator (int index) {
        synchronized (_lock) {
            return SynchronizedLegacyListIterator.create (_inner.listIterator (index), _lock);
        }
    }

    public List subList (int fromIndex, int toIndex) {
        synchronized (_lock) {
            return SynchronizedLegacyList.create (_inner.subList(fromIndex, toIndex), _lock);
        }
    }
}
