package ahc.collection;


/**
 * @author Arno Haase
 */
class SynchronizedLegacyListIterator implements java.util.ListIterator {
    private final java.util.ListIterator _inner;
    private final Object _lock;


    public static SynchronizedLegacyListIterator create (java.util.ListIterator inner, java.util.List lock) {
        if (inner == null)
            return null;

        if (lock == null)
            throw new IllegalArgumentException ("Lock must not be null");

        return new SynchronizedLegacyListIterator (inner, lock);
    }


    private SynchronizedLegacyListIterator (java.util.ListIterator inner, java.util.List lock) {
        _inner = inner;
        _lock = lock;
    }


    public boolean hasNext () {
        synchronized (_lock) {
            return _inner.hasNext();
        }
    }

    public Object next () {
        synchronized (_lock) {
            return _inner.next();
        }
    }

    public boolean hasPrevious () {
        synchronized (_lock) {
            return _inner.hasPrevious();
        }
    }

    public Object previous () {
        synchronized (_lock) {
            return _inner.previous();
        }
    }

    public int nextIndex () {
        synchronized (_lock) {
            return _inner.nextIndex();
        }
    }

    public int previousIndex () {
        synchronized (_lock) {
            return _inner.previousIndex();
        }
    }

    public void remove () {
        synchronized (_lock) {
            _inner.remove ();
        }
    }

    public void set (Object o) {
        synchronized (_lock) {
            _inner.set (o);
        }
    }

    public void add (Object o) {
        synchronized (_lock) {
            _inner.add (o);
        }
    }
}
