package ahc.collection;


/**
 * @author Arno Haase
 */
class SynchronizedList implements List {
    private final List _inner;

    public static SynchronizedList create (List inner) {
        return inner != null ? new SynchronizedList(inner) : null;
    }

    private SynchronizedList (List inner) {
        _inner = inner;
    }

    public synchronized boolean prepend (Object o) {
        synchronized (_inner) {
            return _inner.prepend (o);
        }
    }

    public synchronized boolean append (Object o) {
        synchronized (_inner) {
            return _inner.append (o);
        }
    }

    public synchronized Iterator first (Object o) {
        synchronized (_inner) {
            return SynchronizedIterator.create (_inner.first (o), _inner);
        }
    }

    public synchronized Iterator after () {
        synchronized (_inner) {
            return SynchronizedIterator.create (_inner.after (), _inner);
        }
    }

    public synchronized Iterator end () {
        synchronized (_inner) {
            return SynchronizedIterator.create (_inner.end (), _inner);
        }
    }

    public synchronized Iterator iter (int index) {
        synchronized (_inner) {
            return SynchronizedIterator.create (_inner.iter (index), _inner);
        }
    }

    public synchronized int firstIndexOf (Object o) {
        synchronized (_inner) {
            return _inner.firstIndexOf (o);
        }
    }

    public synchronized Iterator last (Object o) {
        synchronized (_inner) {
            return SynchronizedIterator.create (_inner.last (o), _inner);
        }
    }


    public synchronized Iterator before () {
        synchronized (_inner) {
            return SynchronizedIterator.create (_inner.before (), _inner);
        }
    }

    public synchronized Iterator begin () {
        synchronized (_inner) {
            return SynchronizedIterator.create (_inner.begin (), _inner);
        }
    }


    public synchronized int size () {
        synchronized (_inner) {
            return _inner.size ();
        }
    }

    public synchronized boolean isEmpty () {
        synchronized (_inner) {
            return _inner.isEmpty ();
        }
    }

    public synchronized boolean contains (Object o) {
        synchronized (_inner) {
            return _inner.contains (o);
        }
    }

    public synchronized java.util.Iterator iterator () {
        synchronized (_inner) {
            return SynchronizedLegacyIterator.create (_inner.iterator (), _inner);
        }
    }

    public synchronized Object[] toArray () {
        synchronized (_inner) {
            return _inner.toArray ();
        }
    }

    public synchronized Object[] toArray (Object a[]) {
        synchronized (_inner) {
            return _inner.toArray (a);
        }
    }

    public synchronized boolean add (Object o) {
        synchronized (_inner) {
            return _inner.add (o);
        }
    }

    public synchronized boolean remove (Object o) {
        synchronized (_inner) {
            return _inner.remove (o);
        }
    }

    public synchronized boolean containsAll (java.util.Collection c) {
        synchronized (_inner) {
            return _inner.containsAll (c);
        }
    }

    public synchronized boolean addAll (java.util.Collection c) {
        synchronized (_inner) {
            return _inner.addAll (c);
        }
    }

    public synchronized boolean removeAll (java.util.Collection c) {
        synchronized (_inner) {
            return _inner.removeAll (c);
        }
    }

    public synchronized boolean retainAll (java.util.Collection c) {
        synchronized (_inner) {
            return _inner.retainAll (c);
        }
    }

    public synchronized void clear () {
        synchronized (_inner) {
            _inner.clear ();
        }
    }


    public synchronized boolean addAll (int index, java.util.Collection c) {
        synchronized (_inner) {
            return _inner.addAll (index, c);
        }
    }

    public synchronized Object get (int index) {
        synchronized (_inner) {
            return _inner.get (index);
        }
    }

    public synchronized Object set (int index, Object element) {
        synchronized (_inner) {
            return _inner.set (index, element);
        }
    }

    public synchronized void add (int index, Object element) {
        synchronized (_inner) {
            _inner.add (index, element);
        }
    }

    public synchronized Object remove (int index) {
        synchronized (_inner) {
            return _inner.remove (index);
        }
    }

    public synchronized int indexOf (Object o) {
        synchronized (_inner) {
            return _inner.indexOf (o);
        }
    }

    public synchronized int lastIndexOf (Object o) {
        synchronized (_inner) {
            return _inner.lastIndexOf (o);
        }
    }

    public synchronized java.util.ListIterator listIterator () {
        synchronized (_inner) {
            return SynchronizedLegacyListIterator.create (_inner.listIterator (), this);
        }
    }

    public synchronized java.util.ListIterator listIterator (int index) {
        synchronized (_inner) {
            return SynchronizedLegacyListIterator.create (_inner.listIterator (index), this);
        }
    }

    public synchronized java.util.List subList (int fromIndex, int toIndex) {
        synchronized (_inner) {
            return SynchronizedLegacyList.create (_inner.subList (fromIndex, toIndex), this);
        }
    }


    public synchronized int hashCode () {
        synchronized (_inner) {
            return _inner.hashCode();
        }
    }

    public synchronized boolean equals (Object obj) {
        synchronized (_inner) {
            return _inner.equals (obj);
        }
    }

    public synchronized String toString () {
        synchronized (_inner) {
            return _inner.toString ();
        }
    }
}
