package ahc.collection;

import ahc.collection.equality.EqualityStrategy;


/**
 * @author Arno Haase
 */
class SynchronizedMap implements Map {
    private final Map _inner;


    public static Map create (Map inner) {
        return new SynchronizedMap(inner);
    }


    private SynchronizedMap (Map inner) {
        _inner = inner;
    }


    public EqualityStrategy getValueEqualityStrategy () {
        synchronized (_inner) {
            return _inner.getValueEqualityStrategy();
        }
    }

    public Set keys () {
        synchronized (_inner) {
            return _inner.keys();
        }
    }

    public Collection vals () {
        synchronized (_inner) {
            return _inner.vals ();
        }
    }

    public Set entries () {
        synchronized (_inner) {
            return _inner.entries ();
        }
    }

    public int size () {
        synchronized (_inner) {
            return _inner.size ();
        }
    }

    public boolean isEmpty () {
        synchronized (_inner) {
            return _inner.isEmpty();
        }
    }

    public boolean containsKey (Object key) {
        synchronized (_inner) {
            return _inner.containsKey(key);
        }
    }

    public boolean containsValue (Object value) {
        synchronized (_inner) {
            return _inner.containsValue(value);
        }
    }

    public Object get (Object key) {
        synchronized (_inner) {
            return _inner.get (key);
        }
    }

    public Object put (Object key, Object value) {
        synchronized (_inner) {
            return _inner.put (key, value);
        }
    }

    public Object remove (Object key) {
        synchronized (_inner) {
            return _inner.remove (key);
        }
    }

    public void putAll (java.util.Map t) {
        synchronized (_inner) {
            _inner.putAll (t);
        }
    }

    public void clear () {
        synchronized (_inner) {
            _inner.clear ();
        }
    }

    public java.util.Set keySet () {
        synchronized (_inner) {
            return _inner.keySet();
        }
    }

    public java.util.Collection values () {
        synchronized (_inner) {
            return _inner.values ();
        }
    }

    public java.util.Set entrySet () {
        synchronized (_inner) {
            return _inner.entrySet();
        }
    }


    public int hashCode () {
        synchronized (_inner) {
            return _inner.hashCode();
        }
    }

    public boolean equals (Object obj) {
        synchronized (_inner) {
            return _inner.equals(obj);
        }
    }

    public String toString () {
        synchronized (_inner) {
            return _inner.toString();
        }
    }
}
