package ahc.collection.store;

import ahc.collection.Iterator;
import ahc.collection.equality.EqualityStrategy;


/**
 * @author Arno Haase
 */
abstract class AbstractStore implements Store {
    protected EqualityStrategy _eqStrat;


    public AbstractStore (EqualityStrategy eqStrat) {
        _eqStrat = eqStrat;
    }


    public EqualityStrategy getEqualityStrategy () {
        return _eqStrat;
    }

    public void setEqualityStrategy (EqualityStrategy eqStrat) {
        if (! isEmpty())
            throw new IllegalStateException ("equality strategy can only be changed for empty collections");

        _eqStrat = eqStrat;
    }

    public boolean remove (Object o) {
        final Iterator iter = first (o);
        if (iter == null)
            return false;

        iter.remove ();
        return true;
    }


    public boolean isEmpty () {
        return size () == 0;
    }

    public boolean contains (Object o) {
        return first (o) != null;
    }

    public Iterator first (Object o) {
        for (Iterator iter = before (); iter.hasNext ();) {
            if (_eqStrat.equals (iter.next (), o))
                return iter;
        }

        return null;
    }

    public Iterator last (Object o) {
        for (Iterator iter = after (); iter.hasPrevious (); ) {
            if (_eqStrat.equals (iter.previous (), o))
                return iter;
        }

        return null;
    }


    public Object[] toArray () {
        return toArray (new Object[size ()]);
    }

    public Object[] toArray (Object dest[]) {
        if (dest.length < size ())
            dest = (Object[]) java.lang.reflect.Array.newInstance (dest.getClass ().getComponentType (), size ());

        int ind=0;
        for (Iterator iter = before (); iter.hasNext (); ) {
            dest[ind++] = iter.next ();
        }
        return dest;
    }


    public Object get (int index) {
        return iter (index).current ();
    }

    public Object set (int index, Object element) {
        final Iterator iter = iter (index);
        final Object result = iter.current ();
        iter (index).set (element);
        return result;
    }

    public Object remove (int index) {
        final Iterator iter = iter (index);
        final Object result = iter.current ();
        iter.remove ();
        return result;
    }
}
