package ahc.collection.store;

import ahc.collection.Iterator;
import ahc.collection.equality.EqualityStrategy;


/**
 * A store serves as the internal place where collections store their data.
 *  A store knows nothing about special semantics like sort order, uniqueness of
 *  elements etc. but allows the actual collection efficient access to implement
 *  such aspects.
 *
 * @author Arno Haase
 */
public interface Store {
    /**
     * <tt>true</tt> means that the store retains elements in an order that can
     *  be specified from the outside (e.g. array), <tt>false</tt> otherwise.
     */
    boolean preservesOrdering ();

    /**
     * <tt>true</tt> means that the store's internal representation is sorted (i.e.
     *  relative to a Comparator), <tt>false</tt> otherwise. This is similar to the
     *  issue of order preservation but not the same: e.g. a hash-based store does
     *  not  preserve ordering but is not sorted.
     */
    boolean isSorted ();

    void prepend (Object o);
    void append (Object o);

    Iterator before ();
    Iterator begin ();
    Iterator end ();
    Iterator after ();
    Iterator iter (int n);

    Iterator first (Object o);
    Iterator last (Object o);

    void compactSize ();

    int size();
    boolean isEmpty();
    boolean contains(Object o);
    void clear();

    Object[] toArray();
    Object[] toArray(Object a[]);

    boolean remove(Object o);

    Object get (int index);
    Object set (int index, Object element);
    Object remove (int index);

    EqualityStrategy getEqualityStrategy ();
    void setEqualityStrategy (EqualityStrategy eqStrat);
}
