package ahc.generic;

import ahc.generic.util.AccessorHelper;
import ahc.collection.DoubleKeyCache;
import ahc.collection.TripleKeyCache;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;


/**
 * @author Arno Haase
 */
class CachingSetter implements AttributeSetter {

    private final DoubleKeyCache _getterSetterCache = new DoubleKeyCache () {
        protected Object createNew (Object key1, Object key2) {
            try {
                final Class firstClass = (Class) key1;
                final String accessPath = (String) key2;

                return AccessorHelper.getGetMethodsForSet(firstClass, accessPath);
            } catch (NoSuchMethodException e) {
                throw new IllegalArgumentException ("no such method " + e);
            }
        }
    };

    private final TripleKeyCache _setterCache = new TripleKeyCache () {
        protected Object createNew (Object key1, Object key2, Object key3) {
            try {
                final Class firstClass = (Class) key1;
                final String accessPath = (String) key2;
                final Class paramType = (Class) key3;

                return AccessorHelper.getSetMethod (
                        firstClass,
                        (List) _getterSetterCache.get (firstClass, accessPath),
                        accessPath,
                        paramType);
            } catch (NoSuchMethodException e) {
                throw new IllegalArgumentException ("no such method " + e);
            }
        }
    };


    public void set (Object obj, String accessPath, Object value) {
        try {
            final List getMethods = (List) _getterSetterCache.get(obj.getClass(), accessPath);
            final Class paramType = value != null ? value.getClass() : null;
            final Method setMethod = (Method) _setterCache.get (obj.getClass(), accessPath, paramType);

            final Object lastObject = AccessorHelper.invokeGetMethods(obj, getMethods);
            setMethod.invoke(lastObject, new Object [] {value});
        } catch (IllegalAccessException e) {
            throw new IllegalStateException (e.toString ());
        } catch (InvocationTargetException e) {
            throw new IllegalStateException (e.toString ());
        }
    }

}
