package ahc.generic.classloader;

import ahc.util.bytes.ByteArray;

import java.lang.reflect.Method;


/**
 * @author Arno Haase
 */
abstract class ClassData {
    private final String METHODNAME_CTOR = "<init>";
    private final String SIGNATURE_CTOR = "()V";
    private final String ATTRIBNAME_CODE = "Code";
    private final String CLASSNAME_OBJECT = "java/lang/Object";
    private final ByteArray _data = new ByteArray ();

    protected final ConstantPool _constantPool = new ConstantPool ();

    private final int _indMethodCtor = _constantPool.addMethodInfo (CLASSNAME_OBJECT, METHODNAME_CTOR, SIGNATURE_CTOR);
    private final int _indSuperClass = _constantPool.addClassInfo (CLASSNAME_OBJECT);
    private final int _indMethodnameCtor = _constantPool.addStringInfo (METHODNAME_CTOR);
    private final int _indSigCtor = _constantPool.addStringInfo (SIGNATURE_CTOR);

    protected final int _indAttribCode = _constantPool.addStringInfo (ATTRIBNAME_CODE);

    private final String _clsName;

    private boolean _created = false;


    public ClassData (String clsName) {
        _clsName = clsName;
    }

    private void createData () throws NoSuchMethodException {
        _created = true;

        final int indThisClass = _constantPool.addClassInfo (_clsName);
        int indResolverInterface = getIndSuperclass ();

        final MethodForClassFile getMethod = createMethod ();

        _data.add (new int[]{0xca, 0xfe, 0xba, 0xbe, 0x00, 0x03, 0x00, 0x2d});  // magic, minor, major

        _data.add (_constantPool.asBytes ());

        _data.addInt16 (0x0021); // access flags
        _data.addInt16 (indThisClass);
        _data.addInt16 (_indSuperClass);

        _data.addInt16 (0x0001); // interface_count
        _data.addInt16 (indResolverInterface);

        _data.addInt16 (0); // fields_count

        _data.addInt16 (2); // methods_count
        _data.add (_defaultConstructor.asBytes ());
        _data.add (getMethod.asBytes ());

        _data.addInt16 (0); // attributes_count
    }

    protected abstract int getIndSuperclass ();


    protected final MethodForClassFile _defaultConstructor =
            new DefaultConstructorMethod (_indMethodnameCtor, _indSigCtor, _indAttribCode, _indMethodCtor);


    protected abstract MethodForClassFile createMethod () throws NoSuchMethodException;


    protected static String getClassName (Class cls) {
        return cls.getName ().replace ('.', '/');
    }

    public ByteArray asBytes () throws NoSuchMethodException {
        if (!_created)
            createData ();

        return _data;
    }


    protected int writeGetMethod (final Method curMethod) {
        final String curMethodName = curMethod.getName ();
        final String curClassName = SetterClassData.getClassName (curMethod.getDeclaringClass ());

        final int indMethod = _constantPool.addMethodInfo (curClassName, curMethodName, getGetSignature (curMethod));
        return indMethod;
    }

    private static String getGetSignature (Method method) {
        return "()L" + SetterClassData.getClassName (method.getReturnType ()) + ";";
    }
}
