package ahc.generic.classloader;

import ahc.util.bytes.ByteArray;
import ahc.util.bytes.StringEncoder;
import ahc.util.bytes.StringEndEncoding;
import ahc.collection.Cache;


/**
 * @author Arno Haase
 */
class ConstantPool {
    private static final int CONSTANT_UTF_8 = 1;
    private static final int CONSTANT_Class = 7;
    private static final int CONSTANT_Methodref = 10;
    private static final int CONSTANT_NameAndType = 12;

    private final ByteArray _data = new ByteArray ();
    private int _poolSize = 0;


    private final Cache _stringCache = new Cache () {
        protected Object createNew (Object key) {
            final byte[] raw = StringEncoder.getBytesUtf8 ((String) key, StringEndEncoding.END_LEN_PREFIX_16_HILO);

            _data.addInt8 (CONSTANT_UTF_8);
            _data.add (raw);

            _poolSize++;
            return new Integer (_poolSize);
        }
    };


    public ByteArray asBytes () {
        final ByteArray result = new ByteArray ();

        result.addInt16(_poolSize + 1);
        result.add (_data);

        return result;
    }



    public int addStringInfo (String str) {
        return ((Integer) _stringCache.get(str)).intValue();
    }


    public int addClassInfo (String clsName) {
        final int nameIndex = addStringInfo (clsName);

        _data.addInt8 (CONSTANT_Class);
        _data.addInt16 (nameIndex);

        _poolSize++;
        return _poolSize;
    }

    public int addMethodInfo (String clsName, String methodName, String methodSignature) {
        final int clsIndex = addClassInfo (clsName);
        final int nameAndTypeIndex = addNameAndTypeInfo (methodName, methodSignature);

        _data.addInt8 (CONSTANT_Methodref);
        _data.addInt16 (clsIndex);
        _data.addInt16 (nameAndTypeIndex);

        _poolSize++;
        return _poolSize;
    }

    public int addNameAndTypeInfo (String methodName, String methodSignature) {
        final int nameIndex = addStringInfo (methodName);
        final int descriptorIndex = addStringInfo (methodSignature);

        _data.addInt8 (CONSTANT_NameAndType);
        _data.addInt16 (nameIndex);
        _data.addInt16 (descriptorIndex);

        _poolSize++;
        return _poolSize;
    }
}
