package ahc.generic.classloader;

import ahc.generic.util.AccessorHelper;
import ahc.generic.classloader.internal.SingleAttributeGetter;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.lang.reflect.Method;


/**
 * @author Arno Haase
 */
class GetterClassData extends ClassData {

    private final String METHODNAME_RESOLVER_GETVALUE = "getValue";
    private final String SIGNATURE_OBJECT_TO_OBJECT = "(Ljava/lang/Object;)Ljava/lang/Object;";

    private final Class _firstClass;
    private final String _accessPath;


    public GetterClassData (String clsName, Class clsCast) throws NoSuchMethodException {
        super (clsName);

        _firstClass = clsCast;
        _accessPath = clsName;
    }


    protected MethodForClassFile createMethod () throws NoSuchMethodException {
        final int indClassCast = _constantPool.addClassInfo (getClassName (_firstClass));

        final List getMethods = getGetMethods ();

        final int indMethodnameResolverGetValue = _constantPool.addStringInfo (METHODNAME_RESOLVER_GETVALUE);
        final int indSignatureResolverGetValue = _constantPool.addStringInfo (SIGNATURE_OBJECT_TO_OBJECT);


        final MethodForClassFile getMethod =
                new GetMethod (indMethodnameResolverGetValue, indSignatureResolverGetValue, _indAttribCode,
                        indClassCast, getMethods);
        return getMethod;
    }


    protected List getGetMethods () throws NoSuchMethodException {
        final List result = new ArrayList ();
        final List methods = AccessorHelper.getGetMethods (_firstClass, _accessPath);

        for (Iterator iter = methods.iterator (); iter.hasNext ();) {
            final Method curMethod = (Method) iter.next ();

            int indMethod = writeGetMethod (curMethod);

            result.add (new Integer (indMethod));
        }

        return result;
    }

    protected int getIndSuperclass () {
        return _constantPool.addClassInfo (getClassName (SingleAttributeGetter.class));
    }
}
