package ahc.generic.classloader;

import ahc.util.bytes.ByteArray;


/**
 * @author Arno Haase
 */
abstract class MethodForClassFile {
    private static final int ACCESS_PUBLIC = 0x0001;

    private final int _indName;
    private final int _indSignature;
    private final int _maxStack;
    private final int _numLocals;
    private final int _indCodeAtt;


    protected MethodForClassFile (int indName, int indSignature, int maxStack, int numLocals, int indCodeAtt) {
        _indName = indName;
        _indSignature = indSignature;
        _maxStack = maxStack;
        _numLocals = numLocals;
        _indCodeAtt = indCodeAtt;
    }


    public byte[] asBytes () {
        final ByteArray result = new ByteArray ();

        result.addInt16 (ACCESS_PUBLIC);
        result.addInt16 (_indName);
        result.addInt16 (_indSignature);
        result.addInt16 (1); // attributeCount
        result.addInt16 (_indCodeAtt); // attributes: Code

        final ByteArray body = getBody ();

        result.addInt32 (2 + 2 + 4 + body.getSize() + 2 + 2); // attributeLength: stack, loc vars, code len, code, exc. table, attrib count
        result.addInt16 (_maxStack);
        result.addInt16 (_numLocals);
        result.addInt32 (getBody ().getSize());
        result.add (body);
        result.addInt16 (0); // exception table lenth
        result.addInt16 (0); // attribute count

        return result.asBytes();
    }


    protected abstract ByteArray getBody ();
}
