package ahc.generic.classloader;

import ahc.generic.classloader.internal.SingleAttributeSetter;
import ahc.generic.util.AccessorHelper;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 * @author Arno Haase
 */
class SetterClassData extends ClassData {

    private final String METHODNAME = "setValue";
    private final String SIGNATURE = "(Ljava/lang/Object;Ljava/lang/Object;)V";

    private final Class _firstClass;
    private final String _accessPath;


    public SetterClassData (String clsName, Class clsCast) throws NoSuchMethodException {
        super (clsName);

        _firstClass = clsCast;
        _accessPath = clsName;
    }


    protected MethodForClassFile createMethod () throws NoSuchMethodException {
        final int indClassCast = _constantPool.addClassInfo (getClassName (_firstClass));
        final List getMethods = AccessorHelper.getGetMethodsForSet(_firstClass, _accessPath);

        final List getMethodsInd = getGetMethodsInd (getMethods);
        final Method setMethod = AccessorHelper.getSetMethod(_firstClass, getMethods, _accessPath, null);
        final int indSetMethod = writeSetMethod (setMethod);
        final Class clsParam = setMethod.getParameterTypes()[0];
        final int indParamCast = _constantPool.addClassInfo (getClassName (clsParam));
        final int indMethodname = _constantPool.addStringInfo (METHODNAME);
        final int indSignature = _constantPool.addStringInfo (SIGNATURE);


        final MethodForClassFile result =
                new SetMethod (indMethodname, indSignature, _indAttribCode, indClassCast, indParamCast, getMethodsInd, indSetMethod);
        return result;
    }


    protected List getGetMethodsInd (List methods) throws NoSuchMethodException {
        final List result = new ArrayList ();

        for (Iterator iter = methods.iterator (); iter.hasNext ();) {
            final Method curMethod = (Method) iter.next ();
            int indMethod = writeGetMethod (curMethod);

            result.add (new Integer (indMethod));
        }

        return result;
    }

    protected int getIndSuperclass () {
        return _constantPool.addClassInfo (getClassName (SingleAttributeSetter.class));
    }


    protected int writeSetMethod (final Method curMethod) {
        final String curMethodName = curMethod.getName ();
        final String curClassName = SetterClassData.getClassName (curMethod.getDeclaringClass ());

        final int indMethod = _constantPool.addMethodInfo (curClassName, curMethodName, getSetSignature (curMethod));
        return indMethod;
    }

    private static String getSetSignature (Method method) {
        return "(L" + getClassName (method.getParameterTypes()[0]) + ";)V";
    }
}
