package ahc.generic.util;

import ahc.generic.AttributeGetter;

import java.util.List;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Iterator;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;


/**
 * @author Arno Haase
 */
public class AccessorHelper {

    public static Method getSetMethod (Class firstClass, List getMethods, String accessPath, Class paramType)
            throws NoSuchMethodException {
        final Class cls = getLastClass (firstClass, getMethods);
        final String attName = getAttName (accessPath);
        final String methodName = asSetMethod (attName);

        final Method[] allMethods = cls.getMethods ();
        for (int i = 0; i < allMethods.length; i++) {
            final Method curMethod = allMethods[i];

            if (! curMethod.getName ().equals (methodName))
                continue;
            if (curMethod.getParameterTypes ().length != 1)
                continue;

            if (paramType == null ||
                    curMethod.getParameterTypes ()[0].isAssignableFrom (paramType))
                return curMethod;
        }

        throw new NoSuchMethodException (methodName);
    }

    private static Class getLastClass (Class firstClass, List getMethods) {
        Class cls = firstClass;
        if (getMethods.size() > 0) {
            final Method lastMethod = (Method) getMethods.get(getMethods.size() - 1);
            cls = lastMethod.getReturnType();
        }
        return cls;
    }

    private static String getAttName (String accessPath) {
        String attName;
        final int lastSeparator = accessPath.lastIndexOf (AttributeGetter.CHAR_SEPARATOR);
        if (lastSeparator == -1)
            attName = accessPath;
        else
            attName = accessPath.substring (lastSeparator + 1);
        return attName;
    }


    public static List getGetMethodsForSet (Class firstClass, String accessPath) throws NoSuchMethodException {
        final int lastSeparator = accessPath.lastIndexOf (AttributeGetter.CHAR_SEPARATOR);
        if (lastSeparator == -1) {
            return new ArrayList ();
        }

        return getGetMethods (firstClass, accessPath.substring (0, lastSeparator));
    }


    public static List getGetMethods (Class firstClass, String accessPath) throws NoSuchMethodException {
        final List result = new ArrayList ();
        final StringTokenizer tokenizer = new StringTokenizer (accessPath, AttributeGetter.CHAR_SEPARATOR);

        Class curClass = firstClass;

        while (tokenizer.hasMoreTokens ()) {
            final String curElement = tokenizer.nextToken ();
            final String curMethodName = asGetMethod (curElement);
            final Method curMethod = curClass.getMethod (curMethodName, new Class[]{});

            result.add (curMethod);

            curClass = curMethod.getReturnType ();
        }

        return result;
    }


    public static Object invokeGetMethods (Object obj, List methods) {
        Object result = obj;

        try {
            for (Iterator iter = methods.iterator (); iter.hasNext ();) {
                final Method curMethod = (Method) iter.next ();
                result = curMethod.invoke (result, new Object[0]);
            }
        } catch (IllegalAccessException e) {
            throw new IllegalStateException (e.toString ());
        } catch (InvocationTargetException e) {
            throw new IllegalStateException (e.toString ());
        }

        return result;
    }


    private static String asSetMethod (String attName) {
        return "set" + attName.substring (0, 1).toUpperCase () + attName.substring (1);
    }

    private static String asGetMethod (String attName) {
        return "get" + attName.substring (0, 1).toUpperCase () + attName.substring (1);
    }
}
