package ahc.io;

import java.io.InputStream;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;


/**
 * This class provides an input stream that concatenates one or more other input streams, i.e. first the data in
 *  the first is retured, then the second, then the third etc.
 *
 * @author Arno Haase
 */
public class ConcatInputStream extends InputStream {
    private final List _inner = new ArrayList ();
    private int _currentInd = 0;
    private boolean _closed = false;

    public void add (InputStream is) {
        _inner.add (is);
    }

    private InputStream current () {
        if (_closed)
            throw new IllegalStateException ("Stream is closed.");

        return (InputStream) _inner.get (_currentInd);
    }

    public int read () throws IOException {
        int result = -1;

        while (_currentInd < _inner.size ()) {
            result = current ().read ();
            if (result != -1)
                break;
            _currentInd++;
        }

        return result;
    }

    public boolean markSupported () {
        return false;
    }

    public void close () throws IOException {
        for (Iterator iter = _inner.iterator (); iter.hasNext ();) {
            ((InputStream) iter.next ()).close ();
        }
    }

    public int available () throws IOException {
        if (_currentInd >= _inner.size ())
            return 0;

        return current ().available ();
    }
}
