package ahc.util;


/**
 * @author Arno Haase
 */
public class IntHelper {
    /** an implementation of the mathematically typical - as opposed to the Java - implementation
     *  of the modulus funtion. For "a mod b", this implementation returns a number c such that <br>
     *
     * <ul>
     *   <li> 0 <= c < b, and
     *   <li> a - c is a whole multiple of b,
     * </ul>
     *
     * This differs from the usual Java implementation of a % b which returns negative numbers for
     *  negative a.
     */
    public static int mod (int a, int b) {
        if (b <= 0)
            throw new IllegalArgumentException ("second argument must be positive for mod");

        if (a >= 0)
            return a % b;

        return ((a % b) + b) % b;
    }
}

