package ahc.util.bytes;

import java.util.ArrayList;
import java.util.List;


/**
 * This class serves as an object-oriented interface to a byte array with a couple of convenience methods.
 *
 * @author Arno Haase
 */
public class ByteArray {
    private final List _data = new ArrayList ();

    public ByteArray () {}

    public ByteArray (int[] data) {
        add (data);
    }

    public ByteArray (byte[] data) {
        add (data);
    }

    public ByteArray (ByteArray data) {
        add (data);
    }


    public byte[] asBytes () {
        final byte[] resultBytes = new byte[_data.size ()];
        for (int i = 0; i < _data.size (); i++) {
            resultBytes[i] = (byte) ((Integer) _data.get (i)).intValue ();
        }

        return resultBytes;
    }


    public int getSize () {
        return _data.size();
    }


    public void addInt8 (int num) {
        _data.add (new Integer (BitManipulator.LO (num)));
    }

    public void addInt16 (int num) {
        addInt8 (BitManipulator.MIDLO (num));
        addInt8 (BitManipulator.LO (num));
    }

    public void addInt32 (int num) {
        addInt8 (BitManipulator.HI (num));
        addInt8 (BitManipulator.MIDHI (num));
        addInt8 (BitManipulator.MIDLO (num));
        addInt8 (BitManipulator.LO (num));
    }


    public void add (byte[] bytes) {
        for (int i = 0; i < bytes.length; i++) {
            _data.add (new Integer (bytes[i]));
        }
    }

    public void add (int[] bytes) {
        for (int i = 0; i < bytes.length; i++) {
            _data.add (new Integer (bytes[i]));
        }
    }

    public void add (ByteArray other) {
        add (other.asBytes());
    }
}
