package ahc.util.bytes;

import java.io.UnsupportedEncodingException;


/**
 * This class encodes a string based on a character encoding and a StringEndEncoding, i.e. a way to
 *  represent the length of a string.
 *
 * @author Arno Haase
 */
public class StringEncoder {

    public static byte[] getBytes (String str, String encoding, StringEndEncoding end)
            throws UnsupportedEncodingException {
        final byte[] resultRaw = str.getBytes (encoding);

        return end.addLenEncoding(resultRaw);
    }


    public static byte[] getBytesSafe (String str, String encoding, StringEndEncoding end) {
        try {
            return getBytes (str, encoding, end);
        } catch (UnsupportedEncodingException e) {
            throw new IllegalArgumentException ("Encoding '" + encoding + "' not supported.");
        }
    }


    public static byte[] getBytesUtf8 (String str, StringEndEncoding end) {
            return getBytesSafe (str, "UTF-8", end);
    }

    public static byte[] getBytesIso8859_1 (String str, StringEndEncoding end) {
            return getBytesSafe (str, "iso-8859-1", end);
    }
}
