package ahc.util.bytes;

import ahc.util.bytes.BitManipulator;


/**
 * This class represents ways to encode the length of a string. Three standard implementations are provided
 *  as public static final references.
 *
 * @author Arno Haase
 */
public abstract class StringEndEncoding {

    /**
     * The length of the string is not encoded at all.
     */
    public static final StringEndEncoding END_NONE = new StringEndEncoding () {
        public byte[] addLenEncoding (byte[] raw) {
            return raw;
        }
    };

    /**
     * The length of the string is encoded as a trailing zero byte.
     */
    public static final StringEndEncoding END_NULL = new StringEndEncoding () {
        public byte[] addLenEncoding (byte[] raw) {
            final byte[] result = new byte[raw.length + 1];
            ByteArrayHelper.copy (raw, result, 0);
            result[raw.length] = 0;
            return result;
        }
    };

    /**
     * The length of the string (in bytes) is encoded as a prefixed 16-bit value in hi-lo.
     */
    public static final StringEndEncoding END_LEN_PREFIX_16_HILO = new StringEndEncoding () {
        public byte[] addLenEncoding (byte[] raw) {
            final byte[] result = new byte[raw.length + 2];

            ByteArrayHelper.copy (raw, result, 2);
            result [0] = BitManipulator.MIDLO(raw.length);
            result [1] = BitManipulator.LO(raw.length);

            return result;
        }
    };


    public abstract byte[] addLenEncoding (byte[] raw);
}
