package ahc.util.string;

import java.text.NumberFormat;
import java.text.DateFormat;
import java.util.Date;


/**
 * This class is a collection of helper functions for string handling.
 *
 * @author Arno Haase
 */
public class StringHelper {
    private static final NumberFormat _numFormat = NumberFormat.getNumberInstance();

    /**
     * formats a number using the default locale settings.
     */
    public static String prettyPrint (long num) {
        return _numFormat.format (num);
    }

    /**
     * formats a number using the default locale settings.
     */
    public static String prettyPrint (Number num) {
        return _numFormat.format (num);
    }

    /**
     * formats a date using the default locale settings.
     */
    public static String prettyPrint (Date date) {
        return DateFormat.getDateTimeInstance().format (date);
    }


    /**
     * returns a new string in which one search string is replaced by another.
     */
    public static String replace (String src, String search, String replace) {
        if (src == null)
            return src;
        if (search == null || search.length () == 0)
            throw new IllegalArgumentException ("Search string must not be empty");

        String result = src;
        int ind = 0;
        while ((ind = result.indexOf (search, ind)) >= 0) {
            result = result.substring (0, ind) + replace + result.substring (ind + search.length ());
            ind += replace.length ();
        }

        return result;
    }


    /**
     * replaces special characters that affect formatting with non-formatting character sequences.
     * <ul>
     * <li> \ -> \\
     * <li> &lt;tab&gt; -> \t
     * <li> &lt;CR&gt; -> \r
     * <li> &lt;Newline&gt; -> \n
     * </ul>
     */
    public static String escape (String src) {
        String result = replace (src, "\\", "\\\\");
        result = replace (result, "\t", "\\t");
        result = replace (result, "\r", "\\r");
        result = replace (result, "\n", "\\n");

        return result;
    }

    //todo: unescape

    /**
     * truncates a string regardless of its length. This method is a workaround for a shortcoming of
     *  String.substring (int, int) that is unable to handle the case where the number of characters would
     *  extend beyond the end of the string.
     */
    public static String truncate (String str, int maxLen) {
        if (str == null || str.length () < maxLen)
            return str;
        return str.substring (0, maxLen);
    }
}
