package ahc.util.string;

import ahc.util.string.AttribIterator;

import java.lang.reflect.Field;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;


/**
 * @author Arno Haase
 */
class ToStringDumper extends AttribIterator {

    private final List _result = new ArrayList ();
    private final boolean _escapeStrings;


    public ToStringDumper (boolean filterStatic,
                           boolean filterTransient,
                           boolean alwaysUseToStringImpl,
                           boolean collectionsAsPrimitives,
                           boolean javaLangAsPrimitives,
                           boolean escapeStrings) {
        super (filterStatic, filterTransient, alwaysUseToStringImpl, collectionsAsPrimitives, javaLangAsPrimitives);
        _escapeStrings = escapeStrings;
    }


    protected void handlePre (Object obj) {
        if (obj != null) {
            _result.add ("{" + getSimpleName (obj.getClass ().getName ()) + ": ");
        }
    }

    protected void handlePost (Object obj) {
        if (obj != null) {
            _result.add ("}");
        }
    }

    protected void handlePreCollection (Object o) {
        if (o != null) {
            _result.add ("[");
        }
    }

    protected void handlePostCollection (Object o) {
        if (o != null) {
            _result.add ("]");
        }
    }

    protected void handleNode (Object obj) {
        if (_escapeStrings && obj instanceof String) {
            obj = StringHelper.escape ((String) obj);
        }

        _result.add ("" + obj);
    }

    protected void handleEnumSeparator () {
        _result.add (", ");
    }

    protected void handleMapArrow () {
        _result.add ("->");
    }

    protected void handleField (Field field) {
        _result.add (getSimpleName (field.getName ()) + "=");
    }

    protected void handleCycleDef (Object obj) {
        _result.add (new CycleDefProxy (obj));
    }

    protected void handleCycleRef (Object obj) {
        _result.add ("<" + getCycleNo (obj) + ">");
    }


    protected void handleObject (Object obj) {
        // for cache initialization only
    }

    public String genericToString (Object o, int maxDepth) {
        if (_result.size () > 0)
            throw new IllegalStateException ("Each instance of ToStringDumper can be used only once");

        handleGeneric (o, maxDepth);

        final StringBuffer result = new StringBuffer ();
        for (Iterator iter = _result.iterator (); iter.hasNext ();) {
            result.append (iter.next ());
        }
        return result.toString ();
    }


    String getSimpleName (String fullyQualified) {
        final int lastDot = fullyQualified.lastIndexOf ('.');

        if (lastDot == -1)
            return fullyQualified;

        return fullyQualified.substring (lastDot + 1);
    }

    private class CycleDefProxy {
        private final Object _obj;

        public CycleDefProxy (Object obj) {
            _obj = obj;
        }

        public String toString () {
            if (isCycle (_obj))
                return "<" + getCycleNo (_obj) + ">";

            return "";
        }
    }
}
