package ahc.collection;

import junit.framework.TestCase;

import java.text.Collator;
import java.util.ConcurrentModificationException;


/**
 * @author Arno Haase
 */
public class ListTest extends TestCase {
    public ListTest (String s) {
        super (s);
    }


    private void checkEquals (List l, String expectedRaw) {
        assertEquals (expectedRaw.length () == 0, l.isEmpty ());
        assertEquals (expectedRaw.length (), l.size ());

        final String[] exp = strToArray (expectedRaw);

        for (int i = 0; i < exp.length; i++) {
            assertTrue ("contains " + i, l.contains (exp[i]));
            assertEquals ("equals " + i, exp[i], l.get (i));
        }

        Iterator iter = l.before ();
        loopUp (exp, iter, -1);
        loopDown (exp, iter, exp.length);

        iter = l.begin ();
        loopUp (exp, iter, 0);
        loopDown (exp, iter, exp.length);

        iter = l.end ();
        loopDown (exp, iter, exp.length - 1);
        loopUp (exp, iter, -1);

        iter = l.after ();
        loopDown (exp, iter, exp.length);
        loopUp (exp, iter, -1);
    }

    private void loopUp (String[] exp, Iterator iter, int startInd) {
        for (int i = startInd; i <= exp.length; i++) {
            assertEquals ("hasPrev " + i, i > 0, iter.hasPrevious ());
            assertEquals ("hasCurrent " + i, i >= 0 && i < exp.length, iter.hasCurrent ());
            assertEquals ("hasNext " + i, i < exp.length - 1, iter.hasNext ());

            if (iter.hasCurrent ()) {
                assertEquals ("currentIndex " + i, i, iter.currentIndex ());
                assertEquals ("current " + i, exp[i], iter.current ());
            }

            if (iter.hasPrevious ())
                assertEquals ("previousIndex " + i, i - 1, iter.previousIndex ());

            if (iter.hasNext ()) {
                assertEquals ("nextIndex " + i, i + 1, iter.nextIndex ());
                assertEquals ("next " + i, exp[i + 1], iter.next ());
            }
            else if (i < exp.length)
                iter.next ();
        }
    }

    private void loopDown (String[] exp, Iterator iter, int startInd) {
        for (int i = startInd; i >= -1; i--) {
            assertEquals ("hasPrevious " + i, i > 0, iter.hasPrevious ());
            assertEquals ("hasCurrent " + i, i >= 0 && i < exp.length, iter.hasCurrent ());
            assertEquals ("hasNext " + i, i < exp.length - 1, iter.hasNext ());

            if (iter.hasCurrent ()) {
                assertEquals ("currentIndex " + i, i, iter.currentIndex ());
                assertEquals ("current " + i, exp[i], iter.current ());
            }

            if (iter.hasNext ())
                assertEquals ("nextIndex " + i, i + 1, iter.nextIndex ());

            if (iter.hasPrevious ()) {
                assertEquals ("previousIndex " + i, i - 1, iter.previousIndex ());
                assertEquals ("previous " + i, exp[i - 1], iter.previous ());
            }
            else if (i >= 0)
                iter.previous ();
        }
    }

    private String[] strToArray (String expectedRaw) {
        final String[] result = new String[expectedRaw.length ()];
        for (int i = 0; i < expectedRaw.length (); i++) {
            result[i] = "" + expectedRaw.charAt (i);
        }
        return result;
    }

    public void testDefaultList () {
        _testAllUnsorted (ListHelper.createList ());
    }

    public void testSynchronizedList () {
        _testAllUnsorted (ListHelper.synchronizedList (ListHelper.createList ()));
    }

    public void testArrayList () {
        _testAllUnsorted (ListHelper.createArrayList ());
        _testAllUnsorted (ListHelper.createArrayList (15));

        _testAllSorted (ListHelper.createArrayList (Collator.getInstance ()));
        _testAllSorted (ListHelper.createArrayList (Collator.getInstance (), 1));
        _testAllSorted (ListHelper.createArrayList (Collator.getInstance (), false));
        _testAllSorted (ListHelper.createArrayList (Collator.getInstance (), false, 1));

        _testAllNoDuplicates (ListHelper.createArrayList (null, true));
        _testAllNoDuplicates (ListHelper.createArrayList (null, true, 2));

        _testAllSortedNoDuplicates (ListHelper.createArrayList (Collator.getInstance (), true));
        _testAllSortedNoDuplicates (ListHelper.createArrayList (Collator.getInstance (), true, 3));
    }

    public void testDLinkedList () {
        _testAllUnsorted (ListHelper.createDlinkedList ());

        _testAllSorted (ListHelper.createDlinkedList (Collator.getInstance ()));
        _testAllSorted (ListHelper.createDlinkedList (Collator.getInstance (), false));

        _testAllNoDuplicates (ListHelper.createDlinkedList (null, true));

        _testAllSortedNoDuplicates (ListHelper.createDlinkedList (Collator.getInstance (), true));
    }

    private void _testAllUnsorted (List list) {
        checkEquals (list, "");

        list.add (0, "a");
        checkEquals (list, "a");

        list.append ("c");
        checkEquals (list, "ac");

        list.prepend ("a");
        checkEquals (list, "aac");

        list.add (3, "b");
        checkEquals (list, "aacb");

        list.add ("c");
        checkEquals (list, "aacbc");

        firstLastIndex (list, "a", 0, 1);
        firstLastIndex (list, "c", 2, 4);
        firstLastIndex (list, "b", 3, 3);

        notFound (list, "q");

        assertTrue (list.remove ("c"));
        checkEquals (list, "aabc");

        list.remove (1);
        checkEquals (list, "abc");

        list.first ("b").remove ();
        checkEquals (list, "ac");

        Iterator iter = list.begin ();
        iter.add ("x");
        checkEquals (list, "xac");

        iter.add ("y");
        checkEquals (list, "xyac");

        iter.next ();
        iter.add ("z");
        checkEquals (list, "xyazc");

        iter.next ();
        iter.add ("w");
        checkEquals (list, "xyazcw");

        iter.previous (3);
        iter.replace ("v");
        checkEquals (list, "xyavcw");

        iter.replace ("u");
        checkEquals (list, "xyaucw");

        iter.remove ();
        checkEquals (list, "xyacw");
    }

    private void _testAllNoDuplicates (List list) {
        checkEquals (list, "");

        list.add (0, "a");
        checkEquals (list, "a");

        list.add ("a");
        checkEquals (list, "a");

        list.add (1, "c");
        checkEquals (list, "ac");

        list.add ("b");
        checkEquals (list, "acb");

        list.add ("c");
        checkEquals (list, "acb");

        firstLastIndex (list, "a", 0, 0);
        firstLastIndex (list, "c", 1, 1);
        firstLastIndex (list, "b", 2, 2);

        notFound (list, "q");

        assertTrue (list.remove ("c"));
        checkEquals (list, "ab");

        list.remove (1);
        checkEquals (list, "a");

        list.prepend ("b");
        checkEquals (list, "ba");

        list.append ("d");
        checkEquals (list, "bad");

        list.first ("b").remove ();
        checkEquals (list, "ad");

        Iterator iter = list.begin ();
        iter.add ("x");
        checkEquals (list, "xad");

        iter.add ("y");
        checkEquals (list, "xyad");

        iter.add ("y");
        checkEquals (list, "xyad");

        iter.next ();
        iter.add ("z");
        checkEquals (list, "xyazd");

        iter.next ();
        iter.add ("w");
        checkEquals (list, "xyazdw");

        iter.previous (3);
        iter.replace ("v");
        checkEquals (list, "xyavdw");

        iter.replace ("u");
        checkEquals (list, "xyaudw");

        iter.remove ();
        checkEquals (list, "xyadw");
    }

    private void _testAllSorted (List list) {
        checkEquals (list, "");

        list.add ("c");
        checkEquals (list, "c");

        try {
            list.add (0, "a");
            fail ("Exception expected");
        }
        catch (UnsupportedOperationException exc) {
        }

        list.append ("a");
        checkEquals (list, "ac");

        list.add ("a");
        checkEquals (list, "aac");

        list.prepend ("b");
        checkEquals (list, "aabc");

        list.add ("c");
        checkEquals (list, "aabcc");

        firstLastIndex (list, "a", 0, 1);
        firstLastIndex (list, "c", 3, 4);
        firstLastIndex (list, "b", 2, 2);

        notFound (list, "q");

        assertTrue (list.remove ("c"));
        checkEquals (list, "aabc");

        list.remove (1);
        checkEquals (list, "abc");

        list.first ("b").remove ();
        checkEquals (list, "ac");

        Iterator iter = list.begin ();
        iter.add ("x");
        checkEquals (list, "acx");

        iter.add ("y");
        checkEquals (list, "acxy");

        try {
            iter.replace ("v");
            fail ("Exception expected");
        }
        catch (ConcurrentModificationException exc) {
        }

        try {
            iter.remove ();
            fail ("Exception expected");
        }
        catch (ConcurrentModificationException exc) {
        }
    }

    private void _testAllSortedNoDuplicates (List list) {
        checkEquals (list, "");

        list.add ("a");
        checkEquals (list, "a");

        try {
            list.add (0, "a");
            fail ("Exception expected");
        }
        catch (UnsupportedOperationException exc) {
        }

        list.add ("a");
        checkEquals (list, "a");

        list.prepend ("c");
        checkEquals (list, "ac");

        list.append ("b");
        checkEquals (list, "abc");

        list.add ("c");
        checkEquals (list, "abc");

        firstLastIndex (list, "a", 0, 0);
        firstLastIndex (list, "c", 2, 2);
        firstLastIndex (list, "b", 1, 1);

        notFound (list, "q");

        assertTrue (list.remove ("c"));
        checkEquals (list, "ab");

        list.remove (1);
        checkEquals (list, "a");

        list.prepend ("b");
        checkEquals (list, "ab");

        list.append ("d");
        checkEquals (list, "abd");

        list.first ("b").remove ();
        checkEquals (list, "ad");

        Iterator iter = list.begin ();
        iter.add ("x");
        checkEquals (list, "adx");

        iter.add ("y");
        checkEquals (list, "adxy");

        iter.add ("y");
        checkEquals (list, "adxy");

        try {
            iter.next ();
            fail ("Exception expected");
        }
        catch (ConcurrentModificationException exc) {
        }

        try {
            iter.replace ("v");
            fail ("Exception expected");
        }
        catch (ConcurrentModificationException exc) {
        }

        try {
            iter.remove ();
            fail ("Exception expected");
        }
        catch (ConcurrentModificationException exc) {
        }
    }


    private void notFound (List coll, Object o) {
        assertNull (coll.first (o));
        assertNull (coll.last (o));

        assertEquals (-1, coll.indexOf (o));
        assertEquals (-1, coll.firstIndexOf (o));
        assertEquals (-1, coll.lastIndexOf (o));
    }

    private void firstLastIndex (List coll, Object o, int first, int last) {
        assertEquals (first, coll.first (o).currentIndex ());
        assertEquals (first, coll.indexOf (o));
        assertEquals (first, coll.firstIndexOf (o));
        assertEquals (last, coll.last (o).currentIndex ());
        assertEquals (last, coll.lastIndexOf (o));
    }
}


//todo: Exceptions, insbes. comodification
//todo: EqualityStrategy
