package ahc.collection;

import junit.framework.TestCase;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.text.Collator;

import ahc.util.NaturalOrderComparator;


/**
 * @author Arno Haase
 */
public class MapTest extends TestCase {
    public MapTest (String s) {
        super (s);
    }

    private void checkEqualsUnsorted (String contents, Map map) {
        final java.util.ArrayList reference = new java.util.ArrayList ();
        final java.util.ArrayList refKeys = new java.util.ArrayList ();
        final java.util.ArrayList refValues = new java.util.ArrayList ();
        final java.util.HashMap refMap = new java.util.HashMap ();

        for (int i = 0; i < contents.length (); i += 2) {
            final String key = "" + contents.charAt (i);
            final String value = "" + contents.charAt (i + 1);

            reference.add (new Pair (key, value));
            refKeys.add (key);
            refValues.add (value);

            java.util.ArrayList al = (ArrayList) refMap.get (key);
            if (al == null) {
                al = new java.util.ArrayList ();
                refMap.put (key, al);
            }
            al.add (value);
        }

        Collections.sort (reference, new PairComparator ());

        assertEquals (reference.isEmpty (), map.isEmpty ());
        assertEquals (reference.size (), map.size ());

        assertEqualsColl (refKeys, map.keySet ());
        assertEqualsColl (refKeys, map.keys ());
        assertEqualsColl (refValues, map.values ());
        assertEqualsColl (refValues, map.vals ());

        for (java.util.Iterator iter = refMap.keySet ().iterator (); iter.hasNext ();) {
            final Object curKey = iter.next ();
            final java.util.Collection values = (java.util.Collection) refMap.get (curKey);
            assertTrue (values.contains (map.get (curKey)));

            assertTrue (map.keys ().contains (curKey));
            assertTrue (map.keySet ().contains (curKey));
            assertFalse (map.keys ().contains (curKey + "asdf"));
            assertFalse (map.keySet ().contains (curKey + "asdf"));

            assertTrue (map.values ().contains (map.get (curKey)));
            assertTrue (map.vals ().contains (map.get (curKey)));
            assertFalse (map.values ().contains (map.get (curKey) + "asdf"));
            assertFalse (map.vals ().contains (map.get (curKey) + "asdf"));

            assertTrue (map.entrySet ().contains (new MapEntryImpl (curKey, map.get (curKey))));
            assertTrue (map.entries ().contains (new MapEntryImpl (curKey, map.get (curKey))));
            assertFalse (map.entrySet ().contains (new MapEntryImpl (curKey + "asdf", map.get (curKey))));
            assertFalse (map.entries ().contains (new MapEntryImpl (curKey + "asdf", map.get (curKey))));
            assertFalse (map.entrySet ().contains (new MapEntryImpl (curKey, "asdf" + map.get (curKey))));
            assertFalse (map.entries ().contains (new MapEntryImpl (curKey, "asdf" + map.get (curKey))));
        }

        assertEquals (reference, mapFromEntries (map.entrySet ()));
        assertEquals (reference, mapFromEntries (map.entries ()));
    }

    private java.util.ArrayList mapFromEntries (java.util.Set entries) {
        final java.util.ArrayList result = new java.util.ArrayList ();
        for (java.util.Iterator iter = entries.iterator (); iter.hasNext ();) {
            final java.util.Map.Entry curEntry = (java.util.Map.Entry) iter.next ();
            result.add (new Pair (curEntry.getKey (), curEntry.getValue ()));
        }

        Collections.sort (result, new PairComparator ());
        return result;
    }

    static void assertEqualsColl (java.util.Collection coll1, java.util.Collection coll2) {
        if (!equalsColl (coll1, coll2))
            fail ("expected " + coll1 + ", was " + coll2);
    }

    static void assertEqualsList (java.util.Collection coll1, java.util.Collection coll2) {
        if (!equalsList (coll1, coll2))
            fail ("expected " + coll1 + ", was " + coll2);
    }

    static void assertEqualsSet (java.util.Collection coll1, java.util.Collection coll2) {
        if (!equalsSet (coll1, coll2))
            fail ("expected " + coll1 + ", was " + coll2);
    }

    static boolean equalsColl (java.util.Collection coll1, java.util.Collection coll2) {
        if (coll1 == null)
            return coll2 == null;

        if (coll2 == null)
            return false;

        final ArrayList sorted1 = new ArrayList (coll1);
        final ArrayList sorted2 = new ArrayList (coll2);
        Collections.sort (sorted1);
        Collections.sort (sorted2);

        return sorted1.equals (sorted2);
    }

    static boolean equalsList (java.util.Collection coll1, java.util.Collection coll2) {
        if (coll1 == null)
            return coll2 == null;

        if (coll2 == null)
            return false;

        return new java.util.ArrayList (coll1).equals (new java.util.ArrayList (coll2));
    }

    static boolean equalsSet (java.util.Collection coll1, java.util.Collection coll2) {
        if (coll1 == null)
            return coll2 == null;

        if (coll2 == null)
            return false;

        return new java.util.HashSet (coll1).equals (new java.util.HashSet (coll2));
    }

    private void _testAllUnique (Map map) {
        checkEqualsUnsorted ("", map);

        map.put ("a", "x");
        checkEqualsUnsorted ("ax", map);
        map.remove ("a");
        checkEqualsUnsorted ("", map);

        map.put ("b", "x");
        checkEqualsUnsorted ("bx", map);
        map.put ("c", "y");
        checkEqualsUnsorted ("bxcy", map);

        map.put ("b", "z");
        checkEqualsUnsorted ("bzcy", map);
        map.put ("d", "v");
        checkEqualsUnsorted ("bzcydv", map);
        map.put ("c", "z");
        checkEqualsUnsorted ("bzczdv", map);

        map.remove ("d");
        checkEqualsUnsorted ("bzcz", map);

        map.put ("0", "0");
        map.put ("1", "1");
        map.put ("2", "2");
        map.put ("3", "3");
        map.put ("4", "4");
        map.put ("5", "5");
        map.put ("6", "6");
        map.put ("7", "7");
        map.put ("8", "8");
        map.put ("9", "9");

        map.put ("a", "a");
        map.put ("b", "b");
        map.put ("c", "c");
        map.put ("d", "d");
        map.put ("e", "e");
        map.put ("f", "f");
        map.put ("g", "g");
        map.put ("h", "h");
        map.put ("i", "i");
        map.put ("j", "j");
        map.put ("k", "k");
        map.put ("l", "l");
        map.put ("m", "m");
        map.put ("n", "n");
        map.put ("o", "o");
        map.put ("p", "p");
        map.put ("q", "q");
        map.put ("r", "r");
        map.put ("s", "s");
        map.put ("t", "t");
        map.put ("u", "u");
        map.put ("v", "v");
        map.put ("w", "w");
        map.put ("x", "x");
        map.put ("y", "y");
        map.put ("z", "z");

        checkEqualsUnsorted ("00112233445566778899qqwweerrttzzuuiiooppllkkjjhhggffddssaayyxxccvvbbnnmm", map);

        assertTrue (map.containsKey ("t"));
        assertTrue (map.containsValue ("t"));

        assertFalse (map.containsKey ("asdf"));
        assertFalse (map.containsValue ("asdf"));

        assertNull (map.get ("abc"));
        assertNull (map.get (null));

        map.put ("a", null);
        map.put (null, "123");

        assertNull (map.get ("a"));
        assertEquals ("123", map.get (null));

        map.clear ();
        checkEqualsUnsorted ("", map);
    }

    private void _testAllMulti (Map map) {
        checkEqualsUnsorted ("", map);

        map.put ("a", "x");
        checkEqualsUnsorted ("ax", map);
        map.remove ("a");
        checkEqualsUnsorted ("", map);

        map.put ("b", "x");
        checkEqualsUnsorted ("bx", map);
        map.put ("c", "y");
        checkEqualsUnsorted ("bxcy", map);

        map.put ("b", "z");
        checkEqualsUnsorted ("bxcybz", map);
        map.put ("d", "v");
        checkEqualsUnsorted ("bxcybzdv", map);
        map.put ("c", "z");
        checkEqualsUnsorted ("bxcybzdvcz", map);

        map.remove ("d");
        checkEqualsUnsorted ("bxcybzcz", map);

        map.remove ("b");
        checkEqualsUnsorted ("cycz", map);

        map.clear ();

        map.put ("0", "0");
        map.put ("1", "1");
        map.put ("2", "2");
        map.put ("3", "3");
        map.put ("4", "4");
        map.put ("5", "5");
        map.put ("6", "6");
        map.put ("7", "7");
        map.put ("8", "8");
        map.put ("9", "9");

        map.put ("a", "a");
        map.put ("b", "b");
        map.put ("c", "c");
        map.put ("d", "d");
        map.put ("e", "e");
        map.put ("f", "f");
        map.put ("g", "g");
        map.put ("h", "h");
        map.put ("i", "i");
        map.put ("j", "j");
        map.put ("k", "k");
        map.put ("l", "l");
        map.put ("m", "m");
        map.put ("n", "n");
        map.put ("o", "o");
        map.put ("p", "p");
        map.put ("q", "q");
        map.put ("r", "r");
        map.put ("s", "s");
        map.put ("t", "t");
        map.put ("u", "u");
        map.put ("v", "v");
        map.put ("w", "w");
        map.put ("x", "x");
        map.put ("y", "y");
        map.put ("z", "z");

        checkEqualsUnsorted ("00112233445566778899qqwweerrttzzuuiiooppllkkjjhhggffddssaayyxxccvvbbnnmm", map);

        assertTrue (map.containsKey ("t"));
        assertTrue (map.containsValue ("t"));

        assertFalse (map.containsKey ("asdf"));
        assertFalse (map.containsValue ("asdf"));

        assertNull (map.get ("abc"));
        assertNull (map.get (null));

        map.put ("asdf", null);
        map.put (null, "123");

        assertNull (map.get ("asdf"));
        assertEquals ("123", map.get (null));

        map.clear ();
        checkEqualsUnsorted ("", map);
    }

    public void testSynchronized () {
        _testAllUnique (MapHelper.synchronizedMap (MapHelper.createMap ()));
    }

    public void testDefaultMap () {
        _testAllUnique (MapHelper.createMap ());
    }

    public void testHashMap () {
        _testAllUnique (MapHelper.createHashMap ());
        _testAllUnique (MapHelper.createHashMap (2));
        _testAllUnique (MapHelper.createHashMap (2, 0.8));
        _testAllUnique (MapHelper.createHashMap (MapHelper.ACCESS_UNIQUE));
        _testAllUnique (MapHelper.createHashMap (MapHelper.ACCESS_UNIQUE, 3));
        _testAllUnique (MapHelper.createHashMap (MapHelper.ACCESS_UNIQUE, 3, 0.5));

        _testAllMulti (MapHelper.createHashMap (MapHelper.ACCESS_MULTIPLE));
        _testAllMulti (MapHelper.createHashMap (MapHelper.ACCESS_MULTIPLE, 4));
        _testAllMulti (MapHelper.createHashMap (MapHelper.ACCESS_MULTIPLE, 4, 0.9));
    }

    public void testArrayMap () {
        _testAllUnique (MapHelper.createArrayMap ());
        _testAllUnique (MapHelper.createArrayMap (5));
        _testAllUnique (MapHelper.createArrayMap (Collator.getInstance ()));
        _testAllUnique (MapHelper.createArrayMap (Collator.getInstance (), 5));
        _testAllUnique (MapHelper.createArrayMap (MapHelper.ACCESS_UNIQUE));
        _testAllUnique (MapHelper.createArrayMap (MapHelper.ACCESS_UNIQUE, 2));

        _testAllMulti (MapHelper.createArrayMap (MapHelper.ACCESS_MULTIPLE));
        _testAllMulti (MapHelper.createArrayMap (MapHelper.ACCESS_MULTIPLE, 3));
    }

    public void testDlinkedMap () {
        _testAllUnique (MapHelper.createDlinkedMap ());
        _testAllUnique (MapHelper.createDlinkedMap (Collator.getInstance ()));
        _testAllUnique (MapHelper.createDlinkedMap (MapHelper.ACCESS_UNIQUE));

        _testAllMulti (MapHelper.createDlinkedMap (MapHelper.ACCESS_MULTIPLE));
    }


    static class MapEntryImpl implements Map.Entry {
        private final Object _key, _value;

        public MapEntryImpl (Object key, Object value) {
            _key = key;
            _value = value;
        }

        public Object getKey () {
            return _key;
        }

        public Object getValue () {
            return _value;
        }

        public Object setValue (Object value) {
            throw new UnsupportedOperationException ();
        }
    }


    static class PairComparator implements Comparator {

        public int compare (Object o1, Object o2) {
            final Pair pair1 = (Pair) o1;
            final Pair pair2 = (Pair) o2;

            final int keyResult = NaturalOrderComparator.INSTANCE.compare (pair1.getFirst (), pair2.getFirst ());
            if (keyResult != 0)
                return keyResult;

            return NaturalOrderComparator.INSTANCE.compare (pair1.getSecond (), pair2.getSecond ());
        }
    }
}


//todo: equality strategy
//todo: sorted
