package ahc.collection;

import junit.framework.TestCase;

import java.text.Collator;
import java.util.Collections;


/**
 * @author Arno Haase
 */
public class MultiMapTest extends TestCase {
    public MultiMapTest (String s) {
        super (s);
    }

    private void checkEqualsUnsorted (String contents, MultiMap map) {
        final java.util.HashSet refKeys = new java.util.HashSet ();
        final java.util.ArrayList refValues = new java.util.ArrayList ();
        final java.util.HashMap refMap = new java.util.HashMap ();

        for (int i = 0; i < contents.length (); i += 2) {
            final String key = "" + contents.charAt (i);
            final String value = "" + contents.charAt (i + 1);

            refKeys.add (key);
            refValues.add (value);

            java.util.ArrayList al = (java.util.ArrayList) refMap.get (key);
            if (al == null) {
                al = new java.util.ArrayList ();
                refMap.put (key, al);
            }
            al.add (value);
        }

        assertEquals (refValues.isEmpty (), map.isEmpty ());
        assertEquals (refValues.size (), map.size ());

        assertEqualsColl (refKeys, map.keySet ());
        assertEqualsColl (refKeys, map.keys ());
        assertEqualsColl (refValues, map.values ());
        assertEqualsColl (refValues, map.vals ());

        for (java.util.Iterator iter = refMap.keySet ().iterator (); iter.hasNext ();) {
            final Object curKey = iter.next ();
            final java.util.Collection values = (java.util.Collection) refMap.get (curKey);
            assertTrue (values.equals (map.get (curKey)));

            assertEquals (map.get (curKey), map.getAll (curKey));

            assertEquals (((java.util.Collection) refMap.get (curKey)).size (), map.getCount (curKey));

            assertTrue (map.keys ().contains (curKey));
            assertTrue (map.keySet ().contains (curKey));
            assertFalse (map.keys ().contains (curKey + "asdf"));
            assertFalse (map.keySet ().contains (curKey + "asdf"));

            assertTrue (map.values ().containsAll ((java.util.Collection) map.get (curKey)));
            assertTrue (map.vals ().containsAll ((java.util.Collection) map.get (curKey)));
            assertFalse (map.values ().contains ("asdf"));
            assertFalse (map.vals ().contains ("asdf"));

            assertTrue (map.entrySet ().contains (new MapTest.MapEntryImpl (curKey, map.get (curKey))));
            assertTrue (map.entries ().contains (new MapTest.MapEntryImpl (curKey, map.get (curKey))));
            assertFalse (map.entrySet ().contains (new MapTest.MapEntryImpl (curKey + "asdf", map.get (curKey))));
            assertFalse (map.entries ().contains (new MapTest.MapEntryImpl (curKey + "asdf", map.get (curKey))));
            assertFalse (map.entrySet ().contains (new MapTest.MapEntryImpl (curKey, "asdf" + map.get (curKey))));
            assertFalse (map.entries ().contains (new MapTest.MapEntryImpl (curKey, "asdf" + map.get (curKey))));
        }

        assertEquals (refMap.entrySet (), map.entrySet ());
        assertEquals (refMap.entrySet (), map.entries ());
    }

    static void assertEqualsColl (java.util.Collection coll1, java.util.Collection coll2) {
        if (!equalsColl (coll1, coll2))
            fail ("expected " + coll1 + ", was " + coll2);
    }

    static void assertEqualsList (java.util.Collection coll1, java.util.Collection coll2) {
        if (!equalsList (coll1, coll2))
            fail ("expected " + coll1 + ", was " + coll2);
    }

    static void assertEqualsSet (java.util.Collection coll1, java.util.Collection coll2) {
        if (!equalsSet (coll1, coll2))
            fail ("expected " + coll1 + ", was " + coll2);
    }

    static boolean equalsColl (java.util.Collection coll1, java.util.Collection coll2) {
        if (coll1 == null)
            return coll2 == null;

        if (coll2 == null)
            return false;

        final java.util.ArrayList sorted1 = new java.util.ArrayList (coll1);
        final java.util.ArrayList sorted2 = new java.util.ArrayList (coll2);
        Collections.sort (sorted1);
        Collections.sort (sorted2);

        return sorted1.equals (sorted2);
    }

    static boolean equalsList (java.util.Collection coll1, java.util.Collection coll2) {
        if (coll1 == null)
            return coll2 == null;

        if (coll2 == null)
            return false;

        return new java.util.ArrayList (coll1).equals (new java.util.ArrayList (coll2));
    }

    static boolean equalsSet (java.util.Collection coll1, java.util.Collection coll2) {
        if (coll1 == null)
            return coll2 == null;

        if (coll2 == null)
            return false;

        return new java.util.HashSet (coll1).equals (new java.util.HashSet (coll2));
    }

    private void _testAll (MultiMap map) {
        checkEqualsUnsorted ("", map);

        map.put ("a", "x");
        checkEqualsUnsorted ("ax", map);
        map.remove ("a");
        checkEqualsUnsorted ("", map);

        map.put ("b", "x");
        checkEqualsUnsorted ("bx", map);
        map.put ("c", "y");
        checkEqualsUnsorted ("bxcy", map);

        map.put ("b", "z");
        checkEqualsUnsorted ("bxcybz", map);
        map.put ("d", "v");
        checkEqualsUnsorted ("bxcybzdv", map);
        map.put ("c", "z");
        checkEqualsUnsorted ("bxcybzdvcz", map);

        map.remove ("d");
        checkEqualsUnsorted ("bxcybzcz", map);

        map.remove ("b");
        checkEqualsUnsorted ("cycz", map);

        assertEquals (0, map.getCount ("abcd"));

        map.put ("a", "1");
        checkEqualsUnsorted ("a1cycz", map);

        final MultiMap mm2 = MultiMapHelper.createHashMultiMap ();
        mm2.put ("a", "2");
        mm2.put ("a", "3");
        mm2.put ("a", "4");
        mm2.put ("b", "x");
        mm2.put ("b", "y");
        mm2.put ("c", "z");
        mm2.put ("d", "a");
        map.putAll (mm2);
        checkEqualsUnsorted("a1a2a3a4bxbycyczczda", map);

        map.remove ("a", "3");
        checkEqualsUnsorted("a1a2a4bxbycyczczda", map);
        map.remove ("a");
        checkEqualsUnsorted("bxbycyczczda", map);

        map.clear ();

        map.put ("0", "0");
        map.put ("1", "1");
        map.put ("2", "2");
        map.put ("3", "3");
        map.put ("4", "4");
        map.put ("5", "5");
        map.put ("6", "6");
        map.put ("7", "7");
        map.put ("8", "8");
        map.put ("9", "9");

        map.put ("a", "a");
        map.put ("b", "b");
        map.put ("c", "c");
        map.put ("d", "d");
        map.put ("e", "e");
        map.put ("f", "f");
        map.put ("g", "g");
        map.put ("h", "h");
        map.put ("i", "i");
        map.put ("j", "j");
        map.put ("k", "k");
        map.put ("l", "l");
        map.put ("m", "m");
        map.put ("n", "n");
        map.put ("o", "o");
        map.put ("p", "p");
        map.put ("q", "q");
        map.put ("r", "r");
        map.put ("s", "s");
        map.put ("t", "t");
        map.put ("u", "u");
        map.put ("v", "v");
        map.put ("w", "w");
        map.put ("x", "x");
        map.put ("y", "y");
        map.put ("z", "z");

        checkEqualsUnsorted ("00112233445566778899qqwweerrttzzuuiiooppllkkjjhhggffddssaayyxxccvvbbnnmm", map);

        assertTrue (map.containsKey ("t"));
        assertTrue (map.containsValue ("t"));

        assertFalse (map.containsKey ("asdf"));
        assertFalse (map.containsValue ("asdf"));

        assertNull (map.get ("abc"));
        assertNull (map.get (null));

        map.put ("asdf", null);
        map.put (null, "123");

        assertTrue (((Collection) map.get ("asdf")).contains (null));
        assertEquals ("123", ((List) map.get (null)).get (0));

        map.clear ();
        checkEqualsUnsorted ("", map);
    }

    public void testDefaultMap () {
        _testAll (MultiMapHelper.createMultiMap ());
    }

    public void testHashMap () {
        _testAll (MultiMapHelper.createHashMultiMap ());
        _testAll (MultiMapHelper.createHashMultiMap (2));
        _testAll (MultiMapHelper.createHashMultiMap (2, 0.8));
        _testAll (MultiMapHelper.createHashMultiMap (MapHelper.ACCESS_UNIQUE));
        _testAll (MultiMapHelper.createHashMultiMap (MapHelper.ACCESS_UNIQUE, 3));
        _testAll (MultiMapHelper.createHashMultiMap (MapHelper.ACCESS_UNIQUE, 3, 0.5));

        _testAll (MultiMapHelper.createHashMultiMap (MapHelper.ACCESS_MULTIPLE));
        _testAll (MultiMapHelper.createHashMultiMap (MapHelper.ACCESS_MULTIPLE, 4));
        _testAll (MultiMapHelper.createHashMultiMap (MapHelper.ACCESS_MULTIPLE, 4, 0.9));
    }

    public void testArrayMap () {
        _testAll (MultiMapHelper.createArrayMultiMap ());
        _testAll (MultiMapHelper.createArrayMultiMap (5));
        _testAll (MultiMapHelper.createArrayMultiMap (Collator.getInstance ()));
        _testAll (MultiMapHelper.createArrayMultiMap (Collator.getInstance (), 5));
        _testAll (MultiMapHelper.createArrayMultiMap (MapHelper.ACCESS_UNIQUE));
        _testAll (MultiMapHelper.createArrayMultiMap (MapHelper.ACCESS_UNIQUE, 2));

        _testAll (MultiMapHelper.createArrayMultiMap (MapHelper.ACCESS_MULTIPLE));
        _testAll (MultiMapHelper.createArrayMultiMap (MapHelper.ACCESS_MULTIPLE, 3));
    }

    public void testDlinkedMap () {
        _testAll (MultiMapHelper.createDlinkedMultiMap ());
        _testAll (MultiMapHelper.createDlinkedMultiMap (Collator.getInstance ()));
        _testAll (MultiMapHelper.createDlinkedMultiMap (MapHelper.ACCESS_UNIQUE));

        _testAll (MultiMapHelper.createDlinkedMultiMap (MapHelper.ACCESS_MULTIPLE));
    }


    static class MapEntryImpl implements Map.Entry {
        private final Object _key, _value;

        public MapEntryImpl (Object key, Object value) {
            _key = key;
            _value = value;
        }

        public Object getKey () {
            return _key;
        }

        public Object getValue () {
            return _value;
        }

        public Object setValue (Object value) {
            throw new UnsupportedOperationException ();
        }
    }
}


//todo: equality strategy
//todo: sorted

//todo: putAll, getAll