package ahc.collection;

import junit.framework.TestCase;

import java.util.HashSet;
import java.text.Collator;


/**
 * @author Arno Haase
 */
public class MultiSetTest extends TestCase {
    public MultiSetTest (String s) {
        super (s);
    }


    private void checkEquals (MultiSet set, String expectedRaw) {
        assertEquals (expectedRaw.length () == 0, set.isEmpty ());
        assertEquals (expectedRaw.length (), set.size ());

        final java.util.Set exp = strToSet (expectedRaw);

        for (java.util.Iterator iter = exp.iterator (); iter.hasNext ();) {
            final Object cur = iter.next ();
            assertTrue ("contains " + cur, set.contains (cur));
        }

        final Iterator iter = set.before ();
        assertFalse (iter.hasCurrent ());
        assertEquals (expectedRaw.length () > 0, iter.hasNext ());
        iter.next ();
        loopUp (exp, iter);

        loopUp (exp, set.begin ());
    }

    private void loopUp (java.util.Set exp, Iterator iter) {
        final java.util.Set localCopy = new HashSet (exp);

        for (int i = 0; i < exp.size (); i++) {
            assertEquals ("hasCurrent " + i, i >= 0 && i < exp.size (), iter.hasCurrent ());
            assertEquals ("hasNext " + i, i < exp.size () - 1, iter.hasNext ());

            if (iter.hasCurrent ()) {
                assertTrue (exp.contains (iter.current ()));
                localCopy.remove (iter.current ());
            }

            iter.next ();
        }

        assertTrue (localCopy.isEmpty ());
    }

    private java.util.Set strToSet (String expectedRaw) {
        final HashSet result = new HashSet ();
        for (int i = 0; i < expectedRaw.length (); i++) {
            result.add ("" + expectedRaw.charAt (i));
        }
        return result;
    }

    public void testDefaultSet () {
        _testAll (MultiSetHelper.createMultiSet ());
    }

    public void testArraySet () {
        _testAll (MultiSetHelper.createArrayMultiSet ());
        _testAll (MultiSetHelper.createArrayMultiSet (1));
        _testAll (MultiSetHelper.createArrayMultiSet (Collator.getInstance ()));
        _testAll (MultiSetHelper.createArrayMultiSet (Collator.getInstance (), 2));
    }

    public void testDlinkedSet () {
        _testAll (MultiSetHelper.createDlinkedMultiSet ());
        _testAll (MultiSetHelper.createDlinkedMultiSet (Collator.getInstance ()));
    }

    public void testHashSet () {
        _testAll (MultiSetHelper.createHashMultiSet ());
        _testAll (MultiSetHelper.createHashMultiSet (1));
        _testAll (MultiSetHelper.createHashMultiSet (1, 0.8));
    }


    private void _testAll (MultiSet set) {
        checkEquals (set, "");

        assertEquals (0, set.getCount ("a"));

        set.add ("a");
        checkEquals (set, "a");
        assertEquals (1, set.getCount ("a"));

        set.add ("a");
        checkEquals (set, "a");
        assertEquals (2, set.getCount ("a"));

        set.add ("c");
        checkEquals (set, "ac");
        assertEquals (2, set.getCount ("a"));
        assertEquals (0, set.getCount ("b"));
        assertEquals (1, set.getCount ("c"));

        set.add ("b");
        checkEquals (set, "acb");
        assertEquals (2, set.getCount ("a"));
        assertEquals (1, set.getCount ("b"));
        assertEquals (1, set.getCount ("c"));

        set.add ("c");
        checkEquals (set, "acb");
        assertEquals (2, set.getCount ("a"));
        assertEquals (1, set.getCount ("b"));
        assertEquals (2, set.getCount ("c"));

        found (set, "a");
        found (set, "c");
        found (set, "b");

        notFound (set, "q");

        assertTrue (set.removeSingle ("c"));
        checkEquals (set, "abc");
        assertEquals (2, set.getCount ("a"));
        assertEquals (1, set.getCount ("b"));
        assertEquals (1, set.getCount ("c"));

        assertTrue (set.removeSingle ("c"));
        checkEquals (set, "ab");
        assertEquals (2, set.getCount ("a"));
        assertEquals (1, set.getCount ("b"));
        assertEquals (0, set.getCount ("c"));

        assertFalse (set.removeSingle ("c"));
        checkEquals (set, "ab");
        assertEquals (2, set.getCount ("a"));
        assertEquals (1, set.getCount ("b"));
        assertEquals (0, set.getCount ("c"));

        set.remove ("a");
        checkEquals (set, "b");
        assertEquals (0, set.getCount ("a"));
        assertEquals (1, set.getCount ("b"));
        assertEquals (0, set.getCount ("c"));

        set.add ("a");
        checkEquals (set, "ab");
        assertEquals (1, set.getCount ("a"));
        assertEquals (1, set.getCount ("b"));
        assertEquals (0, set.getCount ("c"));

        set.add ("d");
        checkEquals (set, "bad");
    }


    private void notFound (MultiSet coll, Object o) {
        assertFalse (coll.contains (o));
    }

    private void found (MultiSet coll, Object o) {
        assertTrue (coll.contains (o));
    }
}


//todo: Exceptions, insbes. comodification
//todo: EqualityStrategy
//todo: sorted sets!
//todo: null as value
//todo: removeAll, removeAllSingle