package ahc.collection;

import junit.framework.TestCase;

import java.util.HashSet;
import java.text.Collator;


/**
 * @author Arno Haase
 */
public class SetTest extends TestCase {
    public SetTest (String s) {
        super (s);
    }


    private void checkEquals (Set set, String expectedRaw) {
        assertEquals (expectedRaw.length () == 0, set.isEmpty ());
        assertEquals (expectedRaw.length (), set.size ());

        final java.util.Set exp = strToSet (expectedRaw);

        for (java.util.Iterator iter = exp.iterator (); iter.hasNext ();) {
            final Object cur = iter.next ();
            assertTrue ("contains " + cur, set.contains (cur));
        }

        final Iterator iter = set.before ();
        assertFalse (iter.hasCurrent ());
        assertEquals (expectedRaw.length () > 0, iter.hasNext ());
        iter.next ();
        loopUp (exp, iter);

        loopUp (exp, set.begin ());
    }

    private void loopUp (java.util.Set exp, Iterator iter) {
        final java.util.Set localCopy = new java.util.HashSet (exp);

        for (int i = 0; i < exp.size (); i++) {
            assertEquals ("hasCurrent " + i, i >= 0 && i < exp.size (), iter.hasCurrent ());
            assertEquals ("hasNext " + i, i < exp.size () - 1, iter.hasNext ());

            if (iter.hasCurrent ()) {
                assertTrue (exp.contains (iter.current ()));
                localCopy.remove (iter.current ());
            }

            iter.next ();
        }

        assertTrue (localCopy.isEmpty ());
    }

    private java.util.Set strToSet (String expectedRaw) {
        final java.util.HashSet result = new HashSet ();
        for (int i = 0; i < expectedRaw.length (); i++) {
            result.add ("" + expectedRaw.charAt (i));
        }
        return result;
    }

    public void testDefaultSet () {
        _testAll (SetHelper.createSet ());
    }

    public void testArraySet () {
        _testAll (SetHelper.createArraySet ());
        _testAll (SetHelper.createArraySet (1));
        _testAll (SetHelper.createArraySet (Collator.getInstance ()));
        _testAll (SetHelper.createArraySet (Collator.getInstance (), 2));
    }

    public void testDlinkedSet () {
        _testAll (SetHelper.createDlinkedSet ());
        _testAll (SetHelper.createDlinkedSet (Collator.getInstance ()));
    }

    public void testHashSet () {
        _testAll (SetHelper.createHashSet ());
        _testAll (SetHelper.createHashSet (1));
        _testAll (SetHelper.createHashSet (1, 0.8));
    }


    private void _testAll (Set set) {
        checkEquals (set, "");

        set.add ("a");
        checkEquals (set, "a");

        set.add ("a");
        checkEquals (set, "a");

        set.add ("c");
        checkEquals (set, "ac");

        set.add ("b");
        checkEquals (set, "acb");

        set.add ("c");
        checkEquals (set, "acb");

        found (set, "a");
        found (set, "c");
        found (set, "b");

        notFound (set, "q");

        assertTrue (set.remove ("c"));
        checkEquals (set, "ab");

        set.remove ("b");
        checkEquals (set, "a");

        set.add ("b");
        checkEquals (set, "ba");

        set.add ("d");
        checkEquals (set, "bad");
    }


    private void notFound (Set coll, Object o) {
        assertFalse (coll.contains (o));
    }

    private void found (Set coll, Object o) {
        assertTrue (coll.contains (o));
    }
}


//todo: Exceptions, insbes. comodification
//todo: EqualityStrategy
//todo: sorted sets!
//todo: null as value