package ahc.generic;

import junit.framework.TestCase;

import java.util.Date;


/**
 * @author Arno Haase
 */
public abstract class AttributeGetterTestCase extends TestCase {
    public AttributeGetterTestCase (String s) {
        super (s);
    }


    public void testProblems () {
        final AttributeGetter attribGetter = createAttributeGetter();
        final Person person = createPerson ();

        assertEquals ("Arno", attribGetter.get(person, "name"));

        try {
            attribGetter.get(person, "a/b/c");
            fail ("Exception expected");
        }
        catch (IllegalArgumentException exc) {}

        try {
            attribGetter.get(person, "a#+-");
            fail ("Exception expected");
        }
        catch (IllegalArgumentException exc) {}
    }


    public void testGet () {
        final AttributeGetter attribGetter = createAttributeGetter();
        final Person person = createPerson ();

        assertEquals ("Arno", attribGetter.get (person, "name"));
        assertEquals ("Arno", attribGetter.get (person, "name"));
        assertEquals (new Date (12345), attribGetter.get (person, "birthday"));
        assertEquals (person.getAddress (), attribGetter.get (person, "address"));

        assertEquals ("street 1", attribGetter.get (person, "address/street"));
        assertEquals ("city 1", attribGetter.get (person, "address/city"));

        person.setName ("Marina");
        person.getAddress ().setStreet ("street 2");

        assertEquals ("Marina", attribGetter.get (person, "name"));
        assertEquals ("street 2", attribGetter.get (person, "address/street"));
    }


    protected abstract AttributeGetter createAttributeGetter ();


    public static Person createPerson () {
        final Person person = new Person ("Arno", new Date (12345));
        person.setAddress (new Address ("street 1", "city 1"));
        return person;
    }


    public static class Person {
        private String _name;
        private Date _birthday;
        private Address _address = new Address ();

        public Person (String name, Date birthday) {
            _name = name;
            _birthday = birthday;
        }

        public String getName () {
            return _name;
        }

        public void setName (String name) {
            _name = name;
        }

        public Date getBirthday () {
            return _birthday;
        }

        public void setBirthday (Date birthday) {
            _birthday = birthday;
        }

        public Address getAddress () {
            return _address;
        }

        public void setAddress (Address address) {
            _address = address;
        }

        public String toString () {
            return "Person [" + _name + ", " + _birthday + ", " + _address + "]";
        }
    }

    public static class Address {
        private String _street;
        private String _city;


        public Address () {
        }

        public Address (String street, String city) {
            _street = street;
            _city = city;
        }

        public String getStreet () {
            return _street;
        }

        public void setStreet (String street) {
            _street = street;
        }

        public String getCity () {
            return _city;
        }

        public void setCity (String city) {
            _city = city;
        }


        public String toString () {
            return "Address [" + _street + ", " + _city + "]";
        }
    }
}
