package ahc.generic;

import junit.framework.TestCase;

import java.util.Date;


/**
 * @author Arno Haase
 */
public abstract class AttributeSetterTestCase extends TestCase {
    public AttributeSetterTestCase (String s) {
        super (s);
    }


    public void testProblems () {
        final AttributeSetter attribSetter = createAttributeSetter ();
        final AttributeGetterTestCase.Person person = AttributeGetterTestCase.createPerson ();

        try {
            attribSetter.set (person, "a/b/c", null);
            fail ("Exception expected");
        } catch (IllegalArgumentException exc) {
        }

        try {
            attribSetter.set (person, "a#+-", null);
            fail ("Exception expected");
        } catch (IllegalArgumentException exc) {
        }
    }


    public void testSet () {
        final AttributeSetter attribSetter = createAttributeSetter ();
        final AttributeGetterTestCase.Person person = AttributeGetterTestCase.createPerson ();

        assertEquals ("Arno", person.getName ());
        attribSetter.set (person, "name", "Marina");
        assertEquals ("Marina", person.getName ());
        attribSetter.set (person, "name", "Marina");
        attribSetter.set (person, "name", "Marina");
        assertEquals ("Marina", person.getName ());

        assertEquals (new Date (12345), person.getBirthday ());
        attribSetter.set (person, "birthday", new Date (98765));
        assertEquals (new Date (98765), person.getBirthday ());

        assertEquals ("street 1", person.getAddress ().getStreet ());
        attribSetter.set (person, "address/street", "street 2");
        assertEquals ("street 2", person.getAddress ().getStreet ());

        assertEquals ("city 1", person.getAddress ().getCity ());
        attribSetter.set (person, "address/city", "city 2");
        assertEquals ("city 2", person.getAddress ().getCity ());

        final AttributeGetterTestCase.Address newAddr = new AttributeGetterTestCase.Address ("a", "b");
        attribSetter.set (person, "address", newAddr);
        assertEquals (newAddr, person.getAddress ());
    }


    private void checkSuccess (AttributeSetter setter, String attribute, Object value) {
        final S o = new S ();
        setter.set (o, attribute, value);
        assertTrue (o.getValue () != null);
    }

    private void checkFailure (AttributeSetter setter, String attribute, Object value) {
        final S o = new S ();
        try {
            setter.set (o, attribute, value);
            fail ("Exception expected");
        } catch (IllegalArgumentException exc) {
        }
    }

    private void checkNull (AttributeSetter setter, String attribute) {
        final S o = new S ();
        o.setA (new A ());

        setter.set (o, attribute, null);
        assertNull (o.getValue ());
    }


    public void testSpecial () {
        final AttributeSetter setter = createAttributeSetter ();

        checkSuccess (setter, "a", new A ());
        checkSuccess (setter, "a", new B ());
        checkSuccess (setter, "a", new E ());

        checkFailure (setter, "b", new A ());
        checkSuccess (setter, "b", new B ());
        checkSuccess (setter, "b", new E ());

        checkFailure (setter, "c", new A ());
        checkFailure (setter, "c", new B ());
        checkSuccess (setter, "c", new E ());

        checkFailure (setter, "d", new A ());
        checkFailure (setter, "d", new B ());
        checkSuccess (setter, "d", new E ());

        checkFailure (setter, "e", new A ());
        checkFailure (setter, "e", new B ());
        checkSuccess (setter, "e", new E ());

        checkNull (setter, "a");
        checkNull (setter, "b");
        checkNull (setter, "c");
        checkNull (setter, "d");
        checkNull (setter, "e");
    }

    protected abstract AttributeSetter createAttributeSetter ();


    public static class S {
        private Object _o;

        Object getValue () {
            return _o;
        }

        public void setA (A a) {
            _o = a;
        }

        public void setB (B b) {
            _o = b;
        }

        public void setC (C c) {
            _o = c;
        }

        public void setD (D d) {
            _o = d;
        }

        public void setE (E e) {
            _o = e;
        }
    }

    public static class A {
    }

    public static class B extends A {
    }

    public static interface C {
    }

    public static abstract class D extends B implements C {
    }

    public static class E extends D {
    }
}
