package ahc.util;

import junit.framework.TestCase;

import java.text.DateFormat;
import java.text.SimpleDateFormat;


/**
 * @author Arno Haase
 */
public class DateDayTest extends TestCase {
    public DateDayTest (String s) {
        super (s);
    }

    public void testLeapYear () {
        assertTrue (DateDay.isLeapYear (1996));
        assertFalse (DateDay.isLeapYear (1997));
        assertFalse (DateDay.isLeapYear (1998));
        assertFalse (DateDay.isLeapYear (1999));
        assertTrue (DateDay.isLeapYear (2000));

        assertFalse (DateDay.isLeapYear (1900));
        assertFalse (DateDay.isLeapYear (2100));
    }

    public void testCreate () throws Exception {
        final DateFormat df = new SimpleDateFormat ("dd.MM.yyyy");
        assertEquals (DateDay.create (1, 1, 2000), DateDay.create (df.parse ("1.1.2000")));
        assertEquals (DateDay.create (2, 1, 2000), DateDay.create (df.parse ("2.1.2000")));
        assertEquals (DateDay.create (1, 2, 2000), DateDay.create (df.parse ("1.2.2000")));
        assertEquals (DateDay.create (29, 2, 2000), DateDay.create (df.parse ("29.2.2000")));
        assertEquals (DateDay.create (31, 12, 2000), DateDay.create (df.parse ("31.12.2000")));

        assertEquals (1, DateDay.create (1, 2, 2000).getDay ());
        assertEquals (2, DateDay.create (1, 2, 2000).getMonth ());
        assertEquals (2000, DateDay.create (1, 2, 2000).getYear ());

        assertNull (DateDay.create (null));
    }

    public void testAsDate () {
        final DateFormat df = new SimpleDateFormat ("dd.MM.yyyy");

        assertEquals ("01.01.2003", df.format (DateDay.create (1, 1, 2003).asDate ()));
        assertEquals ("29.02.2000", df.format (DateDay.create (29, 2, 2000).asDate ()));

    }

    public void testCompareTo () {
        assertTrue (DateDay.create (3, 1, 2003).before (DateDay.create (4, 1, 2003)));
        assertTrue (DateDay.create (3, 1, 2003).after (DateDay.create (2, 1, 2003)));

        assertTrue (DateDay.create (5, 4, 2003).after (DateDay.create (30, 3, 2003)));
        assertTrue (DateDay.create (5, 4, 2003).before (DateDay.create (1, 5, 2003)));

        assertTrue (DateDay.create (1, 1, 2004).after (DateDay.create (31, 3, 2003)));
        assertTrue (DateDay.create (31, 3, 2003).before (DateDay.create (1, 1, 2004)));
    }

    public void testIncYear () {
        assertEquals (DateDay.create (1, 1, 2003), DateDay.create (1, 1, 2002).incYear (true));
        assertEquals (DateDay.create (1, 1, 2003), DateDay.create (1, 1, 2002).incYear (false));

        assertEquals (DateDay.create (1, 3, 2005), DateDay.create (29, 2, 2004).incYear (true));
        assertEquals (DateDay.create (28, 2, 2005), DateDay.create (29, 2, 2004).incYear (false));

        assertEquals (DateDay.create (29, 2, 2008), DateDay.create (29, 2, 2004).incYear (4, true));
        assertEquals (DateDay.create (29, 2, 2008), DateDay.create (29, 2, 2004).incYear (4, false));

        assertEquals (DateDay.create (1, 3, 2009), DateDay.create (29, 2, 2004).incYear (5, true));
        assertEquals (DateDay.create (28, 2, 2009), DateDay.create (29, 2, 2004).incYear (5, false));
    }

    public void testDecYear () {
        assertEquals (DateDay.create (1, 1, 2002), DateDay.create (1, 1, 2003).decYear (true));
        assertEquals (DateDay.create (1, 1, 2002), DateDay.create (1, 1, 2003).decYear (false));

        assertEquals (DateDay.create (1, 3, 2003), DateDay.create (29, 2, 2004).decYear (true));
        assertEquals (DateDay.create (28, 2, 2003), DateDay.create (29, 2, 2004).decYear (false));

        assertEquals (DateDay.create (29, 2, 2004), DateDay.create (29, 2, 2008).decYear (4, true));
        assertEquals (DateDay.create (29, 2, 2004), DateDay.create (29, 2, 2008).decYear (4, false));

        assertEquals (DateDay.create (1, 3, 2003), DateDay.create (29, 2, 2008).decYear (5, true));
        assertEquals (DateDay.create (28, 2, 2003), DateDay.create (29, 2, 2008).decYear (5, false));
    }

    public void testDecMonth () {
        assertEquals (DateDay.create (1, 1, 2003), DateDay.create (1, 2, 2003).decMonth (true));
        assertEquals (DateDay.create (1, 1, 2003), DateDay.create (1, 2, 2003).decMonth (false));
        assertEquals (DateDay.create (10, 2, 2003), DateDay.create (10, 3, 2003).decMonth (true));
        assertEquals (DateDay.create (10, 2, 2003), DateDay.create (10, 3, 2003).decMonth (false));

        assertEquals (DateDay.create (17, 12, 2002), DateDay.create (17, 1, 2003).decMonth (true));
        assertEquals (DateDay.create (17, 12, 2002), DateDay.create (17, 1, 2003).decMonth (false));

        assertEquals (DateDay.create (1, 3, 2003), DateDay.create (31, 3, 2003).decMonth (true));
        assertEquals (DateDay.create (28, 2, 2003), DateDay.create (31, 3, 2003).decMonth (false));

        assertEquals (DateDay.create (1, 3, 2004), DateDay.create (31, 3, 2004).decMonth (true));
        assertEquals (DateDay.create (29, 2, 2004), DateDay.create (31, 3, 2004).decMonth (false));

        assertEquals (DateDay.create (5, 8, 2004), DateDay.create (5, 12, 2004).decMonth (4, true));
        assertEquals (DateDay.create (5, 8, 2004), DateDay.create (5, 12, 2004).decMonth (4, false));
        assertEquals (DateDay.create (5, 8, 2004), DateDay.create (5, 1, 2005).decMonth (5, true));
        assertEquals (DateDay.create (5, 8, 2004), DateDay.create (5, 1, 2005).decMonth (5, false));

        assertEquals (DateDay.create (1, 3, 2003), DateDay.create (30, 1, 2005).decMonth (23, true));
        assertEquals (DateDay.create (28, 2, 2003), DateDay.create (30, 1, 2005).decMonth (23, false));

        assertEquals (DateDay.create (1, 3, 2004), DateDay.create (30, 1, 2006).decMonth (23, true));
        assertEquals (DateDay.create (29, 2, 2004), DateDay.create (30, 1, 2006).decMonth (23, false));
    }

    public void testIncMonth () {
        assertEquals (DateDay.create (1, 2, 2003), DateDay.create (1, 1, 2003).incMonth (true));
        assertEquals (DateDay.create (1, 2, 2003), DateDay.create (1, 1, 2003).incMonth (false));
        assertEquals (DateDay.create (10, 3, 2003), DateDay.create (10, 2, 2003).incMonth (true));
        assertEquals (DateDay.create (10, 3, 2003), DateDay.create (10, 2, 2003).incMonth (false));

        assertEquals (DateDay.create (17, 1, 2003), DateDay.create (17, 12, 2002).incMonth (true));
        assertEquals (DateDay.create (17, 1, 2003), DateDay.create (17, 12, 2002).incMonth (false));

        assertEquals (DateDay.create (1, 3, 2003), DateDay.create (31, 1, 2003).incMonth (true));
        assertEquals (DateDay.create (28, 2, 2003), DateDay.create (31, 1, 2003).incMonth (false));

        assertEquals (DateDay.create (1, 3, 2004), DateDay.create (31, 1, 2004).incMonth (true));
        assertEquals (DateDay.create (29, 2, 2004), DateDay.create (31, 1, 2004).incMonth (false));

        assertEquals (DateDay.create (5, 12, 2004), DateDay.create (5, 8, 2004).incMonth (4, true));
        assertEquals (DateDay.create (5, 12, 2004), DateDay.create (5, 8, 2004).incMonth (4, false));
        assertEquals (DateDay.create (5, 1, 2005), DateDay.create (5, 8, 2004).incMonth (5, true));
        assertEquals (DateDay.create (5, 1, 2005), DateDay.create (5, 8, 2004).incMonth (5, false));

        assertEquals (DateDay.create (1, 3, 2003), DateDay.create (30, 1, 2001).incMonth (25, true));
        assertEquals (DateDay.create (28, 2, 2003), DateDay.create (30, 1, 2001).incMonth (25, false));

        assertEquals (DateDay.create (1, 3, 2004), DateDay.create (30, 1, 2001).incMonth (37, true));
        assertEquals (DateDay.create (29, 2, 2004), DateDay.create (30, 1, 2001).incMonth (37, false));
    }

    public void testDecDay () {
        assertEquals (DateDay.create (1, 1, 2003), DateDay.create (2, 1, 2003).decDay ());
        assertEquals (DateDay.create (2, 1, 2003), DateDay.create (3, 1, 2003).decDay ());
        assertEquals (DateDay.create (31, 1, 2003), DateDay.create (1, 2, 2003).decDay ());
        assertEquals (DateDay.create (28, 2, 2003), DateDay.create (1, 3, 2003).decDay ());
        assertEquals (DateDay.create (31, 12, 2003), DateDay.create (1, 1, 2004).decDay ());

        assertEquals (DateDay.create (28, 2, 2004), DateDay.create (29, 2, 2004).decDay ());
        assertEquals (DateDay.create (29, 2, 2004), DateDay.create (1, 3, 2004).decDay ());

        assertEquals (DateDay.create (5, 1, 2003), DateDay.create (25, 1, 2003).decDay (20));
        assertEquals (DateDay.create (10, 1, 2003), DateDay.create (10, 2, 2003).decDay (31));
        assertEquals (DateDay.create (10, 12, 2002), DateDay.create (10, 3, 2003).decDay (31 + 31 + 28));
        assertEquals (DateDay.create (10, 12, 2003), DateDay.create (10, 3, 2004).decDay (31 + 31 + 29));
    }

    public void testIncDay () {
        assertEquals (DateDay.create (2, 1, 2003), DateDay.create (1, 1, 2003).incDay ());
        assertEquals (DateDay.create (3, 1, 2003), DateDay.create (2, 1, 2003).incDay ());
        assertEquals (DateDay.create (1, 2, 2003), DateDay.create (31, 1, 2003).incDay ());
        assertEquals (DateDay.create (1, 3, 2003), DateDay.create (28, 2, 2003).incDay ());
        assertEquals (DateDay.create (1, 1, 2004), DateDay.create (31, 12, 2003).incDay ());

        assertEquals (DateDay.create (29, 2, 2004), DateDay.create (28, 2, 2004).incDay ());
        assertEquals (DateDay.create (1, 3, 2004), DateDay.create (29, 2, 2004).incDay ());

        assertEquals (DateDay.create (25, 1, 2003), DateDay.create (5, 1, 2003).incDay (20));
        assertEquals (DateDay.create (10, 2, 2003), DateDay.create (10, 1, 2003).incDay (31));
        assertEquals (DateDay.create (10, 3, 2003), DateDay.create (10, 12, 2002).incDay (31 + 31 + 28));
        assertEquals (DateDay.create (10, 3, 2004), DateDay.create (10, 12, 2003).incDay (31 + 31 + 29));
    }
}
