package ahc.util;

import junit.framework.TestCase;


/**
 * @author Arno Haase
 */
public class IntegerFactoryTest extends TestCase {
    public IntegerFactoryTest (String s) {
        super (s);
    }


    public void testGet () {
        for (int i = -10000; i < 10000; i++) {
            assertEquals (i, IntegerFactory.get (i).intValue ());
            assertSame (IntegerFactory.get (i), IntegerFactory.get (i));
        }
    }

    public void testIncDec () {
        for (int i = -10000; i < 10000; i++) {
            checkSameness (i + 1, IntegerFactory.increment (new Integer (i)));
            checkSameness (i - 1, IntegerFactory.decrement (new Integer (i)));
        }
    }

    private void checkSameness (int expected, Integer i) {
        assertEquals (expected, i.intValue ());
        assertSame (IntegerFactory.get (expected), i);
    }

    public void testAddSubtract () {
        for (int i = -1000; i < 1000; i++) {
            for (int j = -10; j < 10; j++) {
                checkSameness (i + j, IntegerFactory.add (new Integer (i), new Integer (j)));
                checkSameness (i + j, IntegerFactory.add (new Integer (i), j));

                checkSameness (i - j, IntegerFactory.subtract (new Integer (i), new Integer (j)));
                checkSameness (i - j, IntegerFactory.subtract (new Integer (i), j));
                checkSameness (i - j, IntegerFactory.subtract (i, new Integer (j)));
            }
        }
    }

    public void testMod () {
        for (int i = -100; i < 100; i++) {
            try {
                IntegerFactory.mod (i, -1);
                fail ("Exception expected");
            }
            catch (IllegalArgumentException exc) {
            }

            try {
                IntegerFactory.mod (i, 0);
                fail ("Exception expected");
            }
            catch (IllegalArgumentException exc) {
            }

            for (int j = 1; j < 20; j++) {
                checkMod (i, j, IntegerFactory.mod (i, j).intValue ());
                checkMod (i, j, IntegerFactory.mod (IntegerFactory.get (i), j).intValue ());
                checkMod (i, j, IntegerFactory.mod (i, IntegerFactory.get (j)).intValue ());
                checkMod (i, j, IntegerFactory.mod (IntegerFactory.get (i), IntegerFactory.get (j)).intValue ());
            }
        }
    }

    private void checkMod (int i, int j, final int mod) {
        assertTrue ("range " + i + "%" + j + "=" + mod, 0 <= mod && mod < j);
        assertTrue ("value " + i + "%" + j + "=" + mod, (i - mod) % j == 0);
    }
}
