package ahc.util.string;

import junit.framework.TestCase;

import java.util.Arrays;


/**
 * @author Arno Haase
 */
public class HexFormatterTest extends TestCase {
    public HexFormatterTest (String s) {
        super (s);
    }

    public void testToEmptyString () {
        assertEquals ("", HexFormatter.toHexString (new byte[]{}));
        assertEquals ("", HexFormatter.toHexString (new char[]{}));
        assertEquals ("", HexFormatter.toHexString (new short[]{}));
        assertEquals ("", HexFormatter.toHexString (new int[]{}));
        assertEquals ("", HexFormatter.toHexString (new long[]{}));
    }

    public void testToString () {
        assertEquals ("0100FF", HexFormatter.toHexString (new byte[]{1, 0, -1}));
        assertEquals ("00010000FFFF", HexFormatter.toHexString (new char[]{1, 0, 0xFFFF}));
        assertEquals ("00010000FFFF", HexFormatter.toHexString (new short[]{1, 0, -1}));
        assertEquals ("0000000100000000FFFFFFFF", HexFormatter.toHexString (new int[]{1, 0, -1}));
        assertEquals ("00000000000000010000000000000000FFFFFFFFFFFFFFFF",
                HexFormatter.toHexString (new long[]{1, 0, -1}));
    }

    public void testToArrayFromEmptyString () {
        assertTrue (Arrays.equals (new byte[]{}, HexFormatter.toByteArray ("")));
        assertTrue (Arrays.equals (new char[]{}, HexFormatter.toCharArray ("")));
        assertTrue (Arrays.equals (new short[]{}, HexFormatter.toShortArray ("")));
        assertTrue (Arrays.equals (new int[]{}, HexFormatter.toIntArray ("")));
        assertTrue (Arrays.equals (new long[]{}, HexFormatter.toLongArray ("")));
    }

    public void testToArrayUppercaseedString () {
        assertTrue (Arrays.equals (new byte[]{1, 0, -1}, HexFormatter.toByteArray ("0100FF")));
        assertTrue (Arrays.equals (new char[]{1, 0, 0xFFFF}, HexFormatter.toCharArray ("00010000FFFF")));
        assertTrue (Arrays.equals (new short[]{1, 0, -1}, HexFormatter.toShortArray ("00010000FFFF")));
        assertTrue (Arrays.equals (new int[]{1, 0, -1}, HexFormatter.toIntArray ("0000000100000000FFFFFFFF")));
        assertTrue (Arrays.equals (new long[]{1, 0, -1},
                HexFormatter.toLongArray ("00000000000000010000000000000000FFFFFFFFFFFFFFFF")));
    }

    public void testToArrayMixedString () {
        assertTrue (Arrays.equals (new byte[]{-1, -1}, HexFormatter.toByteArray ("fFfF")));
        assertTrue (Arrays.equals (new char[]{0xFFFF, 0xFFFF}, HexFormatter.toCharArray ("fFfFfFfF")));
        assertTrue (Arrays.equals (new short[]{-1, -1}, HexFormatter.toShortArray ("fFfFfFfF")));
        assertTrue (Arrays.equals (new int[]{-1, -1}, HexFormatter.toIntArray ("fFfFfFfFfFfFfFfF")));
        assertTrue (Arrays.equals (new long[]{-1, -1},
                HexFormatter.toLongArray ("fFfFfFfFfFfFfFfFfFfFfFfFfFfFfFfF")));
    }

    public void testErrorCases () {
        try {
            HexFormatter.toByteArray ("0");
            fail ("NumberFormatException must be thrown");
        } catch (NumberFormatException e) {
        }
        try {
            HexFormatter.toByteArray ("xx");
            fail ("NumberFormatException must be thrown");
        } catch (NumberFormatException e) {
        }

        try {
            HexFormatter.toCharArray ("012");
            fail ("NumberFormatException must be thrown");
        } catch (NumberFormatException e) {
        }
        try {
            HexFormatter.toCharArray ("xxxx");
            fail ("NumberFormatException must be thrown");
        } catch (NumberFormatException e) {
        }

        try {
            HexFormatter.toShortArray ("012");
            fail ("NumberFormatException must be thrown");
        } catch (NumberFormatException e) {
        }
        try {
            HexFormatter.toShortArray ("xxxx");
            fail ("NumberFormatException must be thrown");
        } catch (NumberFormatException e) {
        }

        try {
            HexFormatter.toIntArray ("0123456");
            fail ("NumberFormatException must be thrown");
        } catch (NumberFormatException e) {
        }
        try {
            HexFormatter.toIntArray ("xxxxxxxx");
            fail ("NumberFormatException must be thrown");
        } catch (NumberFormatException e) {
        }

        try {
            HexFormatter.toLongArray ("0123456789abcde");
            fail ("NumberFormatException must be thrown");
        } catch (NumberFormatException e) {
        }
        try {
            HexFormatter.toLongArray ("xxxxxxxxxxxxxxxx");
            fail ("NumberFormatException must be thrown");
        } catch (NumberFormatException e) {
        }

    }
}
